@@ -165,6 +165,7 @@
  *	Return Mod of floating point number
  */
 
+#define _BSD_SOURCE
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -179,7 +180,7 @@ static double dint();
 static double dmod();
 static int ndec = 3;
 void
-setdatedec (nd)
+setdatedec (int nd)
 { ndec = nd; return; }
 
 
