@@ -140,7 +140,7 @@ static hdrl_pool * hdrl_pool_mmap_new(size_t pool_size)
     pool->p.size = pool_size;
     pool->p.free_offset = pool->p.base;
     cpl_msg_debug(cpl_func, "Creating mmap pool %p of size %zu",
-                  pool, pool_size);
+                  (const void*)pool, pool_size);
     return (hdrl_pool*)pool;
 }
 
@@ -160,7 +160,7 @@ static hdrl_pool * hdrl_pool_malloc_new(size_t pool_size)
     pool->p.base = cpl_malloc(pool_size);
     pool->p.free_offset = pool->p.base;
     cpl_msg_debug(cpl_func, "Creating malloc pool %p of size %zu",
-                  pool, pool_size);
+                  (const void*)pool, pool_size);
     return (hdrl_pool*)pool;
 }
 
@@ -170,7 +170,7 @@ static void hdrl_pool_delete(hdrl_pool * pool)
         return;
     }
 
-    cpl_msg_debug(cpl_func, "Deleting pool %p", pool);
+    cpl_msg_debug(cpl_func, "Deleting pool %p", (const void*)pool);
     pool->destructor(pool);
     cpl_free(pool);
 }
