@@ -58,6 +58,7 @@
 #include "vmimgutils.h"
 #include "vmcpl.h"
 #include "vmqcutils.h"
+#include "vimos_dfs.h"
 
 
 #define  NSIGMA             (4.)
@@ -425,7 +426,6 @@ vmdet(PilSetOfFrames *sof)
   float        gain[4];                    
   float        delay[4];                    
   float        delay1, delay2;
-  float        raw_delay;
   float       *noise;                   
   float        value, mean_delay, err_delay, mean_gain, err_gain;
   float        mean_noise, err_noise;
@@ -531,7 +531,7 @@ vmdet(PilSetOfFrames *sof)
    *  in twin lists.
    */
 
-  flatList  = (VimosImage **)pil_calloc(flatCount, sizeof(VimosImage *));
+  flatList  = (VimosImage **)cpl_calloc(flatCount, sizeof(VimosImage *));
 
   frame = pilSofLookupNext(sof, flatCategory);
 
@@ -541,7 +541,7 @@ vmdet(PilSetOfFrames *sof)
       cpl_msg_error(task, "Cannot open flat field %s", pilFrmGetName(frame));
       for (j = 0; j < i; j++)
         deleteImage(flatList[j]);
-      pil_free(flatList);
+      cpl_free(flatList);
       return EXIT_FAILURE;
     }
     pilFrmSetType(frame, PIL_FRAME_TYPE_RAW);
@@ -550,9 +550,9 @@ vmdet(PilSetOfFrames *sof)
 
   pairCount = flatCount / 2;
 
-  flatList1 = (VimosImage **)pil_calloc(pairCount, sizeof(VimosImage *));
-  flatList2 = (VimosImage **)pil_calloc(pairCount, sizeof(VimosImage *));
-  expInt    = (int *)pil_calloc(pairCount, sizeof(int));
+  flatList1 = (VimosImage **)cpl_calloc(pairCount, sizeof(VimosImage *));
+  flatList2 = (VimosImage **)cpl_calloc(pairCount, sizeof(VimosImage *));
+  expInt    = (int *)cpl_calloc(pairCount, sizeof(int));
 
   for (i = 0; i < flatCount; i++) {
     if (readFloatDescriptor(flatList[i]->descs, 
@@ -562,10 +562,10 @@ vmdet(PilSetOfFrames *sof)
                   pilTrnGetKeyword("ExposureTime"));
       for (i = 0; i < flatCount; i++)
         deleteImage(flatList[i]);
-      pil_free(flatList);
-      pil_free(flatList1);
-      pil_free(flatList2);
-      pil_free(expInt);
+      cpl_free(flatList);
+      cpl_free(flatList1);
+      cpl_free(flatList2);
+      cpl_free(expInt);
       return EXIT_FAILURE;
     }
 
@@ -581,10 +581,10 @@ vmdet(PilSetOfFrames *sof)
                       expTime);
           for (i = 0; i < flatCount; i++)
             deleteImage(flatList[i]);
-          pil_free(flatList);
-          pil_free(flatList1);
-          pil_free(flatList2);
-          pil_free(expInt);
+          cpl_free(flatList);
+          cpl_free(flatList1);
+          cpl_free(flatList2);
+          cpl_free(expInt);
           return EXIT_FAILURE;
         }
         flatList2[j] = flatList[i];
@@ -597,10 +597,10 @@ vmdet(PilSetOfFrames *sof)
                   "Check the exposure times of the input flat fields.");
       for (i = 0; i < flatCount; i++)
         deleteImage(flatList[i]);
-      pil_free(flatList);
-      pil_free(flatList1);
-      pil_free(flatList2);
-      pil_free(expInt);
+      cpl_free(flatList);
+      cpl_free(flatList1);
+      cpl_free(flatList2);
+      cpl_free(expInt);
       return EXIT_FAILURE;
     }
     
@@ -628,10 +628,10 @@ vmdet(PilSetOfFrames *sof)
   if (error) {
     for (i = 0; i < flatCount; i++)
       deleteImage(flatList[i]);
-    pil_free(flatList);
-    pil_free(flatList1);
-    pil_free(flatList2);
-    pil_free(expInt);
+    cpl_free(flatList);
+    cpl_free(flatList1);
+    cpl_free(flatList2);
+    cpl_free(expInt);
     return EXIT_FAILURE;
   }
 
@@ -650,15 +650,15 @@ vmdet(PilSetOfFrames *sof)
     cpl_msg_error(task, "Failure in growing overscans in master bias");
     for (i = 0; i < flatCount; i++)
       deleteImage(flatList[i]);
-    pil_free(flatList);
-    pil_free(flatList1);
-    pil_free(flatList2);
-    pil_free(expInt);
+    cpl_free(flatList);
+    cpl_free(flatList1);
+    cpl_free(flatList2);
+    cpl_free(expInt);
     deleteImage(biasImage);
     return EXIT_FAILURE;
   }
 
-  pil_free(flatList);
+  cpl_free(flatList);
 
 
   /*
@@ -690,7 +690,7 @@ vmdet(PilSetOfFrames *sof)
    * be used, for reasons of limited memory available.
    */
 
-  keep = (int *)pil_calloc(pairCount, sizeof(int));
+  keep = (int *)cpl_calloc(pairCount, sizeof(int));
 
   if (pairCount <= keep_flat) {
     for (i = 0; i < pairCount; i++)
@@ -734,7 +734,7 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
   /* End of patch for memory limits. To remove it, delete also
    * any reference to keep[]. */
 
-  pil_free(expInt);
+  cpl_free(expInt);
 
 
  /*
@@ -756,9 +756,9 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
     time_flux[i] = newDpoint(pairCount);
   }
 
-  pairMean = (VimosImage **)pil_calloc(pairCount, sizeof(VimosImage *));
-  pair = (VimosImage **)pil_calloc(2, sizeof(VimosImage *));
-  noise = (float *)pil_calloc(flatCount, sizeof(float));
+  pairMean = (VimosImage **)cpl_calloc(pairCount, sizeof(VimosImage *));
+  pair = (VimosImage **)cpl_calloc(2, sizeof(VimosImage *));
+  noise = (float *)cpl_calloc(flatCount, sizeof(float));
 
   newPairCount = 0;
 
@@ -776,20 +776,31 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
     meanImage = frCombAverage(pair, 2);
     copyAllDescriptors(pair[0]->descs, &(meanImage->descs));
 
+    /*
+     * Now go for the bias subtraction
+     * 
+     * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+     * which is the default. Since the overscan region in the master bias has 
+     * been created with growOverscans, there is some extra noise added here,
+     * but this can probably be ignored. A better approach is to use the method
+     * used in the MOS pipeline: subtract overscan for each individual frame,
+     * then subtract bias.
+     */
+
     if (VmSubBias(meanImage, biasImage, BIAS_ZMASTER) == EXIT_FAILURE) {
       for (j = i; j < pairCount; j++)
         deleteImage(flatList1[j]);
-      pil_free(flatList1);
+      cpl_free(flatList1);
       for (j = i; j < pairCount; j++)
         deleteImage(flatList2[j]);
-      pil_free(flatList2);
-      pil_free(pair);
+      cpl_free(flatList2);
+      cpl_free(pair);
       for (j = 0; j < newPairCount; j++)
         deleteImage(pairMean[j]);
-      pil_free(pairMean);
+      cpl_free(pairMean);
       deleteImage(meanImage);
       deleteImage(biasImage);
-      pil_free(keep);
+      cpl_free(keep);
       return EXIT_FAILURE;
     }
 
@@ -826,7 +837,7 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
       flux_variance[j][i].x = medianPixelvalue(sample1, npix);
       time_flux[j][i].y = flux_variance[j][i].x;
   
-      pil_free(sample1);
+      cpl_free(sample1);
   
       sample1 = extractFloatImage(pair[0]->data, pair[0]->xlen, pair[0]->ylen,
                                   winPrscStartX, winPrscStartY, 
@@ -836,7 +847,7 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
                                   winPrscStartX, winPrscStartY, 
                                   winSizeX / 2, winSizeY / 2);
   
-      diff = pil_malloc(npix * sizeof(float));
+      diff = cpl_malloc(npix * sizeof(float));
   
       for (k = 0; k < npix; k++)
         diff[k] = sample1[k] - sample2[k];
@@ -856,9 +867,9 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
         "%5.2f s, %7.1f ADU, %7.1f ADU", 
         i + 1, expTime, flux_variance[j][i].x, flux_variance[j][i].y);
   
-      pil_free(diff);
-      pil_free(sample1);
-      pil_free(sample2);
+      cpl_free(diff);
+      cpl_free(sample1);
+      cpl_free(sample2);
     }
 
     if (keep[i]) {
@@ -877,9 +888,9 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
   }
 
   deleteImage(biasImage);
-  pil_free(pair);
-  pil_free(flatList1);
-  pil_free(flatList2);
+  cpl_free(pair);
+  cpl_free(flatList1);
+  cpl_free(flatList2);
 
 
   /*
@@ -903,10 +914,9 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
   for (j = 0; j < 4; j++) {
     fit = fitSlopeRobust(flux_variance[j], pairCount);
     gain[j] = 1 / fit[1];
-    pil_free(fit);
+    cpl_free(fit);
     fit = fitSlopeRobust(time_flux[j], pairCount);
-    raw_delay = - fit[0] / fit[1];
-    pil_free(fit);
+    cpl_free(fit);
     if (fit[2] > 200) {
       cpl_msg_error(task, "The input data are invalid: either the flat "
                     "lamp was strongly variable during the observation "
@@ -921,7 +931,7 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
              / (fit[1] + sqrt(fit[1] * fit[1] - 4 * fit[2] * fit[0]));
     delay2 = -2 * fit[0] 
              / (fit[1] - sqrt(fit[1] * fit[1] - 4 * fit[2] * fit[0]));
-    pil_free(fit);
+    cpl_free(fit);
     if (fabs(delay1) < fabs(delay2))
       delay[j] = delay1;
     else
@@ -1083,7 +1093,7 @@ printf("keep[%d] = %d (%d)\n", i, keep[i], expInt[i]);
     }
   }
 
-  pil_free(keep);
+  cpl_free(keep);
 
   for (i = 0; i < 4; i++)
     deleteDpoint(flux_variance[i]);
@@ -1146,7 +1156,7 @@ printf("--------------------------------------------------------------\n");
       if (errorImage) 
         errorImage->data[x + y * xlen] = fit[2];
 
-      pil_free(fit);
+      cpl_free(fit);
 
     }
   }
@@ -1260,7 +1270,7 @@ printf("--------------------------------------------------------------\n");
         nIter++;
       }
 
-      pil_free(cell);
+      cpl_free(cell);
 
       /* 
        *  Setting here the user-specified tolerance.  
@@ -1286,7 +1296,7 @@ printf("--------------------------------------------------------------\n");
       flux_average[count].y = average;
       flux_sigma[count].y   = tolerance;
 
-      pil_free(cell);
+      cpl_free(cell);
 
       count++;
     }
@@ -1340,8 +1350,8 @@ printf("--------------------------------------------------------------\n");
   deleteImage(filtered);
 
 
-  pil_free(averageCurve);
-  pil_free(sigmaCurve);
+  cpl_free(averageCurve);
+  cpl_free(sigmaCurve);
 
   deleteDpoint(exposure_flux);
 
@@ -1354,7 +1364,7 @@ printf("--------------------------------------------------------------\n");
     cpl_msg_error(task, "Cannot create CCD table");
     for (i = 0; i < newPairCount; i++)
       deleteImage(pairMean[i]);
-    pil_free(pairMean);
+    cpl_free(pairMean);
     deleteImage(badPixelImage);
     deleteImage(errorImage);
     return EXIT_FAILURE;
@@ -1431,7 +1441,7 @@ printf("--------------------------------------------------------------\n");
       cpl_msg_error(task, "Failure updating product header");
       for (j = 0; j < i; j++)
         deleteImage(pairMean[j]);
-      pil_free(pairMean);
+      cpl_free(pairMean);
       deleteImage(badPixelImage);
       deleteImage(errorImage);
       deleteTable(ccdTable);
@@ -1495,7 +1505,7 @@ printf("--------------------------------------------------------------\n");
    * update the set of frames.
    */
 
-  strlower(strcpy(ccdTableName, ccdTableCategory));
+  vmstrlower(strcpy(ccdTableName, ccdTableCategory));
   /* strcat(ccdTableName, ".TFITS"); */
   strcat(ccdTableName, ".fits");
 
@@ -1519,7 +1529,7 @@ printf("--------------------------------------------------------------\n");
 
     for (i = 0; i < flatCount; i++)
       deleteImage(flatList[i]);
-    pil_free(flatList);
+    cpl_free(flatList);
     deleteImage(badPixelImage);
     deleteImage(errorImage);
     deleteTable(ccdTable);
@@ -1528,7 +1538,7 @@ printf("--------------------------------------------------------------\n");
   }
 
   if (createMap) {
-    strlower(strcpy(badPixelImageName, badPixelCategory));
+    vmstrlower(strcpy(badPixelImageName, badPixelCategory));
     strcat(badPixelImageName, ".fits");
 
     if (createFitsImage(badPixelImageName, badPixelImage, badPixelCategory)) {
@@ -1547,7 +1557,7 @@ printf("--------------------------------------------------------------\n");
 
       for (i = 0; i < flatCount; i++)
         deleteImage(flatList[i]);
-      pil_free(flatList);
+      cpl_free(flatList);
       deleteImage(badPixelImage);
       deleteImage(errorImage);
       deleteTable(ccdTable);
@@ -1572,7 +1582,7 @@ printf("--------------------------------------------------------------\n");
   
       for (i = 0; i < flatCount; i++)
         deleteImage(flatList[i]);
-      pil_free(flatList);
+      cpl_free(flatList);
       deleteImage(badPixelImage);
       deleteImage(errorImage);
       deleteTable(ccdTable);
