@@ -56,6 +56,7 @@
 #include "vmimgphotcalib.h"
 #include "vmimgutils.h"
 #include "vmcpl.h"
+#include "vimos_dfs.h"
 
 
 /*
@@ -603,6 +604,17 @@ vmimpreimaging(PilSetOfFrames *sof)
     biasImage = resizedBias;
   }
     
+  /*
+   * Now go for the bias subtraction
+   * 
+   * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+   * which is the default. Since the overscan region in the master bias has 
+   * been created with growOverscans, there is some extra noise added here,
+   * but this can probably be ignored. A better approach is to use the method
+   * used in the MOS pipeline: subtract overscan for each individual frame,
+   * then subtract bias.
+   */
+
   if (VmSubBias(rawImage, biasImage, biasMethod) == EXIT_FAILURE) {
     cpl_msg_error(task, "Bias correction failed!");
     deleteImage(biasImage);
@@ -821,7 +833,7 @@ vmimpreimaging(PilSetOfFrames *sof)
   * Reduced image
   */
 
-  strlower(strcpy(productName, productTag));
+  vmstrlower(strcpy(productName, productTag));
   strcat(productName, ".fits");
 
 
