@@ -679,6 +679,8 @@ static double get_orientation(cpl_table *table, double *rms)
             mean *= binsize;
             mean -= 90.0;
         }
+        cpl_free(profile);
+        cpl_free(triprofile);
     }
 
     *rms = stdv;
