@@ -1486,6 +1486,17 @@ vmskyccd(PilSetOfFrames *sof)
       biasImage = resizedBias;
     }
     
+    /*
+     * Now go for the bias subtraction
+     * 
+     * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+     * which is the default. Since the overscan region in the master bias has 
+     * been created with growOverscans, there is some extra noise added here,
+     * but this can probably be ignored. A better approach is to use the method
+     * used in the MOS pipeline: subtract overscan for each individual frame,
+     * then subtract bias.
+     */
+
     if (VmSubBias(rawImage, biasImage, biasMethod) == EXIT_FAILURE) {
       cpl_msg_error(task, "Bias correction failed!");
 
@@ -1807,9 +1818,9 @@ vmskyccd(PilSetOfFrames *sof)
 
   allStars = starTable->cols->len;
 
-  star_orig = (int *)pil_calloc(allStars, sizeof(int));
-  ximage_orig = (double *)pil_calloc(allStars, sizeof(double));
-  yimage_orig = (double *)pil_calloc(allStars, sizeof(double));
+  star_orig = (int *)cpl_calloc(allStars, sizeof(int));
+  ximage_orig = (double *)cpl_calloc(allStars, sizeof(double));
+  yimage_orig = (double *)cpl_calloc(allStars, sizeof(double));
 
 
   star_tmp = tblGetIntData(starTable, "NUMBER");
@@ -1859,7 +1870,7 @@ vmskyccd(PilSetOfFrames *sof)
   
   /* Save the star match product */
   starMatchProductTag = (char *)pilTrnGetCategory("ImgStarMatchAstrometry");
-  strlower(strcpy(starMatchProductName, starMatchProductTag));
+  vmstrlower(strcpy(starMatchProductName, starMatchProductTag));
   strcat(starMatchProductName, ".fits");
   if (createFitsTable(starMatchProductName, starMatchTable, 
                       starMatchProductTag) != VM_TRUE) {
@@ -1930,8 +1941,8 @@ vmskyccd(PilSetOfFrames *sof)
   * We recover the original sex coordinates 
   */
 
-  ximage = (double *)pil_calloc(numStars, sizeof(double));
-  yimage = (double *)pil_calloc(numStars, sizeof(double));
+  ximage = (double *)cpl_calloc(numStars, sizeof(double));
+  yimage = (double *)cpl_calloc(numStars, sizeof(double));
 
   for (i = 0; i < numStars; i++) {
     for (j = 0; j < allStars; j++) {
@@ -2196,7 +2207,7 @@ vmskyccd(PilSetOfFrames *sof)
     }
   } 
  
-  pil_free(coeffControlString);
+  cpl_free(coeffControlString);
 
  /*
   * Setting up the output PAF file  
@@ -2231,7 +2242,7 @@ skip:
   */
 
   productTag = reducedTag;
-  strlower(strcpy(productName, productTag));
+  vmstrlower(strcpy(productName, productTag));
   strcat(productName, ".fits");
   insertDoubleDescriptor(&rawImageFF->descs, pilTrnGetKeyword("DataMin"),
                          imageMinimum(rawImageFF), pilTrnGetComment("DataMin"),
@@ -2257,6 +2268,10 @@ skip:
   insertStringDescriptor(&rawImageFF->descs, pilTrnGetKeyword("DoCategory"),
                          productTag, pilTrnGetKeyword("DoCategory"),"ESO*", 1);
   deleteSetOfDescriptors(&rawImageFF->descs, "ESO DPR*");
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(rawImageFF->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(rawImageFF->descs), "GCOUNT");  
     
   if (createFitsImage(productName, rawImageFF, productTag) != VM_TRUE) {
     cpl_msg_error(task, "Cannot create local product file %s!", productName);
