@@ -54,6 +54,7 @@
 #include "vmmossphotcalib.h"
 #include "vmmosutils.h"
 #include "vmcpl.h"
+#include "vimos_dfs.h"
 
 
 static cxint vmspphot(PilSetOfFrames *);
@@ -410,7 +411,7 @@ vmspphot(PilSetOfFrames *sof)
    * Load images
    */
 
-  if ((imaList = (VimosImage **)pil_calloc(imaCount, sizeof(VimosImage *)))) {
+  if ((imaList = (VimosImage **)cpl_calloc(imaCount, sizeof(VimosImage *)))) {
 
     frame = pilSofLookupNext(sof, reducedCategory);
 
@@ -424,7 +425,7 @@ vmspphot(PilSetOfFrames *sof)
                       i + 1);
         for (j = 0; j < i; j++)
           deleteImage(imaList[j]);
-        pil_free(imaList);
+        cpl_free(imaList);
         return EXIT_FAILURE;
       }
       frame = pilSofLookupNext(sof, NULL);
@@ -455,7 +456,7 @@ vmspphot(PilSetOfFrames *sof)
                     pilTrnGetKeyword("Quadrant"));
     for (i = 0; i < imaCount; i++)
       deleteImage(imaList[i]);
-    pil_free(imaList);
+    cpl_free(imaList);
     return EXIT_FAILURE;
   }
 
@@ -478,7 +479,7 @@ vmspphot(PilSetOfFrames *sof)
       cpl_msg_error(task, "Missing grism name in input images.");
     for (i = 0; i < imaCount; i++)
       deleteImage(imaList[i]);
-    pil_free(imaList);
+    cpl_free(imaList);
     return EXIT_FAILURE;
   }
 
@@ -512,7 +513,7 @@ vmspphot(PilSetOfFrames *sof)
   if (error) {
     for (j = 0; j < imaCount; j++)
       deleteImage(imaList[j]);
-    pil_free(imaList);
+    cpl_free(imaList);
     return EXIT_FAILURE;
   }
 
@@ -552,7 +553,7 @@ vmspphot(PilSetOfFrames *sof)
       deleteTable(atmTable);
       for (j = 0; j < imaCount; j++)
         deleteImage(imaList[j]);
-      pil_free(imaList);
+      cpl_free(imaList);
       return EXIT_FAILURE;
     }
 
@@ -565,7 +566,7 @@ vmspphot(PilSetOfFrames *sof)
         deleteTable(sphotTable);
         for (j = 0; j < imaCount; j++)
           deleteImage(imaList[j]);
-        pil_free(imaList);
+        cpl_free(imaList);
         return EXIT_FAILURE;
       }
     }
@@ -581,7 +582,7 @@ vmspphot(PilSetOfFrames *sof)
         deleteTable(sphotTable);
         for (j = 0; j < imaCount; j++)
           deleteImage(imaList[j]);
-        pil_free(imaList);
+        cpl_free(imaList);
         return EXIT_FAILURE;
       }
     }
@@ -601,7 +602,7 @@ vmspphot(PilSetOfFrames *sof)
       deleteTable(sphotTable);
       for (j = 0; j < imaCount; j++)
         deleteImage(imaList[j]);
-      pil_free(imaList);
+      cpl_free(imaList);
       return EXIT_FAILURE;
     }
     deleteImage(imaList[i]);
@@ -657,7 +658,7 @@ vmspphot(PilSetOfFrames *sof)
       cpl_msg_error(task, "Failure updating product %d header", i+1);
       for (j = 0; j < imaCount; j++)
         deleteImage(imaList[j]);
-      pil_free(imaList);
+      cpl_free(imaList);
       return EXIT_FAILURE;
     } 
 
@@ -665,7 +666,7 @@ vmspphot(PilSetOfFrames *sof)
       reducedCategory = fluxReducedCategory;
 
     sprintf(output1DName, "%s_%d.fits", reducedCategory, i);
-    strlower(output1DName);
+    vmstrlower(output1DName);
 
     if (createFitsImage(output1DName, imaList[i], reducedCategory)) {
       outputFrame = newPilFrame(output1DName, reducedCategory);
@@ -681,7 +682,7 @@ vmspphot(PilSetOfFrames *sof)
       cpl_msg_error(task, "Cannot create local product file %s!", output1DName);
       for (j = 0; j < imaCount; j++)
         deleteImage(imaList[j]);
-      pil_free(imaList);
+      cpl_free(imaList);
       return EXIT_FAILURE;
     }
 
@@ -689,7 +690,7 @@ vmspphot(PilSetOfFrames *sof)
 
   for (j = 0; j < imaCount; j++)
     deleteImage(imaList[j]);
-  pil_free(imaList);
+  cpl_free(imaList);
 
   return EXIT_SUCCESS;
 
