@@ -263,17 +263,31 @@ class PlotableMappedScience :
     
     #Reading the object table
     if self.objecttables is not None:
+
       self.nobjects = []
       self.ybottom_obj_extracts = []
       self.ytop_obj_extracts = []
+      self.ybottom_slits = []
+      self.ytop_slits = []
+
       for iext in range(0, len(self.objecttables)) :
         objecttable = self.objecttables[iext]
         nslit = objecttable.getTableNrows(iext+1)
-        n_skip_columns = 10
+
+        n_object_cols = 0
         for col in objecttable.all_hdu[iext+1].data.columns:
-          if(col.name == 'multiplex'):
-            n_skip_columns = 12
-        maxobjectperslit = (objecttable.getTableNcols(iext+1) - n_skip_columns ) / 4
+          if "object" in col.name:
+            n_object_cols = n_object_cols + 1
+
+        maxobjectperslit = n_object_cols
+
+        objecttable.readTableColumn(iext+1, 'position')
+        position = objecttable.column
+        self.ybottom_slits.append(position)
+
+        objecttable.readTableColumn(iext+1, 'length')
+        self.ytop_slits.append([ylow + length for (ylow, length) in zip(position, objecttable.column) if length > 0])
+
         start_extracted_cols = []
         end_extracted_cols = []
         for obj in range(maxobjectperslit):
@@ -303,10 +317,17 @@ class PlotableMappedScience :
                                    self.mappedsciences[self.ext_sel].image)
     if self.objecttables is not None:
       subplot.autoscale(enable=False)
+
       for obj in range(self.nobjects[self.ext_sel]) :
         subplot.axhline(self.ytop_obj_extracts[self.ext_sel][obj], linestyle='solid',color='red')
         subplot.axhline(self.ybottom_obj_extracts[self.ext_sel][obj], linestyle='solid',color='yellow')
-        
+      
+      color_slit = 'blue'
+      for yslit in self.ytop_slits[self.ext_sel]:
+        subplot.axhline(yslit, linestyle='solid', color=color_slit)        
+      for yslit in self.ybottom_slits[self.ext_sel]:
+        subplot.axhline(yslit, linestyle='solid', color=color_slit)        
+
   def getObjectInPosition(self, ypos) :
     for obj in range(self.nobjects[self.ext_sel]) :
       if ypos > self.ybottom_obj_extracts[self.ext_sel][obj] and \
@@ -518,44 +539,64 @@ class PlotableSkylinesOffsets :
                             self.allskylines_wave, self.allwave_res)
 
 class PlotableExtractedScience :
-  def __init__(self, fits_extractedscience):
+
+  def __init__(self, fits_extractedscience, fits_extractedfluxcalibrated = None):
     self.obj_id = -1
     self.ext_sel = 0
-    extractedscience      = PipelineProduct(fits_extractedscience)
+    self.plot_flux_cal = False
+    extractedscience = PipelineProduct(fits_extractedscience)
+    extractedfluxcalibrated = self.getPipelineProduct(fits_extractedfluxcalibrated)
     self.multiext = False
     self.ext_sel = 0
     if len(extractedscience.all_hdu) > 1:
       self.multiext = True
     self.extractedsciences = []
+    self.extractedfluxcalibratedsciences = []
     if(self.multiext) :
       for iext in range(1, len(extractedscience.all_hdu)) :
         self.extractedsciences.append(PipelineProduct(fits_extractedscience))
+        self.extractedfluxcalibratedsciences.append(self.getPipelineProduct(fits_extractedfluxcalibrated))
     else :
         self.extractedsciences.append(extractedscience)
+        self.extractedfluxcalibratedsciences.append(extractedfluxcalibrated)
+
     self.spectrumdisplay   = SpectrumDisplay()
     self.loadFromFits()
 
+  def getPipelineProduct(self, frame):
+    if frame is None:
+      return None
+    return  PipelineProduct(frame)
+
   def loadFromFits(self) :
     #Reading the object images
     if(self.multiext) :
       read_ext = 1
     else:
       read_ext = 0
-      
 
     self.nobj = []
-    self.bunit = []
+    self.bunit_extracted = []
+    self.bunit_flux_cal = []
     self.wave = []
+
     for i in range(0, len(self.extractedsciences)) :
+
       extractedscience = self.extractedsciences[i]
       extractedscience.readImage(read_ext)
+
+      extractedscience_flux_cal = self.extractedfluxcalibratedsciences[i]
+      if extractedscience_flux_cal is not None:
+      	extractedscience_flux_cal.readImage(read_ext)
+	self.bunit_flux_cal.append(extractedscience_flux_cal.readKeyword('BUNIT', 0))
+      
       read_ext = read_ext + 1
 
       self.nobj.append(extractedscience.image.shape[0])
       crpix1  = extractedscience.readKeyword('CRPIX1', 0)
       crval1  = extractedscience.readKeyword('CRVAL1', 0)
       cdelt1  = extractedscience.readKeyword('CD1_1', 0)
-      self.bunit.append(extractedscience.readKeyword('BUNIT', 0))
+      self.bunit_extracted.append(extractedscience.readKeyword('BUNIT', 0))
       nwave  =   extractedscience.image.shape[1]
       self.wave.append(numpy.arange(1, nwave+1, 1))
       self.wave[i] = (self.wave[i] - crpix1) * cdelt1 + crval1
@@ -576,20 +617,35 @@ class PlotableExtractedScience :
   
   def setFluxSelected(self) :
     self.flux = self.extractedsciences[self.ext_sel].image[self.obj_id-1,:]
+    self.flux_cal = None
+    if(self.extractedfluxcalibratedsciences[self.ext_sel] is not None):
+    	self.flux_cal = self.extractedfluxcalibratedsciences[self.ext_sel].image[self.obj_id-1,:]
 
   def selectObject(self, obj_id):
     self.obj_id = obj_id
     self.setFluxSelected()
 
-  def selectMultiplexing(self, multiplexing_idx):
+  def selectMultiplexing(self, multiplexing_idx, plot_flux_cal = False):
     self.ext_sel = multiplexing_idx - 1
     self.selectBrightest()
     self.setFluxSelected()
+    self.plot_flux_cal = plot_flux_cal
 
   def plot(self, subplot, title, tooltip):
 
-    self.spectrumdisplay.setLabels('Lambda', 'Total Flux ['+self.bunit[self.ext_sel]+']')
-    self.spectrumdisplay.display(subplot, title, tooltip, self.wave[self.ext_sel], self.flux,
+    units = self.bunit_extracted[self.ext_sel]
+
+    if(self.plot_flux_cal and self.flux_cal is not None):
+    	units = self.bunit_flux_cal[self.ext_sel]
+
+    self.spectrumdisplay.setLabels('Lambda', 'Total Flux ['+units+']')
+    	    
+    if(not self.plot_flux_cal):
+    	self.spectrumdisplay.display(subplot, title, tooltip, self.wave[self.ext_sel], self.flux,
+                                autolimits = True)
+
+    elif(self.flux_cal is not None):
+    	self.spectrumdisplay.display(subplot, title, tooltip, self.wave[self.ext_sel], self.flux_cal,
                                 autolimits = True)
 
 class PlotableSpecPhot :
@@ -618,6 +674,12 @@ class PlotableSpecPhot :
     self.wave_used = self.wave[self.used_fit > 0]
     self.raw_response_used = self.raw_response[self.used_fit > 0]    
 
+  def getMinWave(self):
+    return min(min(self.wave_nonnull), min(self.wave_used))
+
+  def getMaxWave(self):
+    return max(max(self.wave_nonnull), max(self.wave_used))
+
   def plotResponse(self, subplot, title, tooltip):
     self.respdisp.setLabels('$\lambda\, [\AA]$','$10^{-16} erg\, cm^{-2} e-^{-1}$')
     self.respdisp.flux_lim = 0., numpy.max(self.raw_response_nonnull) * 1.1
