@@ -747,16 +747,16 @@ cpl_table *dfs_load_table(cpl_frameset *frameset, const char *category, int ext)
  * where the failure occurred is also reported.
  */
 /*----------------------------------------------------------------------------*/
-cpl_propertylist *dfs_load_header(cpl_frameset *frameset, 
+cpl_propertylist *dfs_load_header(const cpl_frameset *frameset,
                                   const char *category, int ext)
 {
     const char *func = "dfs_load_header";
 
-    cpl_frame         *frame = NULL;
+    const cpl_frame   *frame = NULL;
     cpl_propertylist  *plist = NULL;
 
 
-    frame = cpl_frameset_find(frameset, category);
+    frame = cpl_frameset_find_const(frameset, category);
 
     if (frame) {
         plist = cpl_propertylist_load(cpl_frame_get_filename(frame), ext);
@@ -916,6 +916,48 @@ int dfs_save_image(cpl_frameset *frameset, const cpl_image *image,
  * @return 0 in case of success.
  *
  * This function is just a wrapper to the basic CPL functions
+ * that are routinely called every time a table product must
+ * be saved to disk by a recipe. Error checking and proper
+ * messaging are also included here, to give a more readable
+ * look to the main recipe code.
+ *
+ * The output file name will be derived from the specified
+ * category by lowercasing it and by appending the suffix ".fits".
+ * The new table is properly logged in the input set-of-frames
+ * in case of success.
+ *
+ * The error codes that are set in this case are the same set
+ * by the above mentioned CPL functions. The "where" string
+ * (accessible via a call to @c cpl_error_get_where() ) is
+ * not modified by this function, and therefore the function
+ * where the failure occurred is also reported.
+ */
+/*----------------------------------------------------------------------------*/
+int dfs_save_table(cpl_frameset *frameset, const cpl_table *table,
+                   const char *category, cpl_propertylist *header,
+                   const cpl_parameterlist *parlist, const char *recipename,
+                   const char *version){
+	return dfs_save_table_with_header_ext(frameset, table, category, header,
+			NULL, parlist, recipename, version);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief
+ *   Saving table data of given category.
+ *
+ * @param frameset   The input set-of-frames (to be upgraded)
+ * @param table      The table to save
+ * @param category   The category of the table to save
+ * @param header     Header to input to cpl_dfs_setup_product_header()
+ * @param ext_header Header for the extension where the table is written
+ * @param parlist    The recipe parameter list
+ * @param recipename The name of the recipe
+ * @param version    The version of the pipeline
+ *
+ * @return 0 in case of success.
+ *
+ * This function is just a wrapper to the basic CPL functions
  * that are routinely called every time a table product must 
  * be saved to disk by a recipe. Error checking and proper 
  * messaging are also included here, to give a more readable 
@@ -933,8 +975,9 @@ int dfs_save_image(cpl_frameset *frameset, const cpl_image *image,
  * where the failure occurred is also reported.
  */
 /*----------------------------------------------------------------------------*/
-int dfs_save_table(cpl_frameset *frameset, const cpl_table *table, 
+int dfs_save_table_with_header_ext(cpl_frameset *frameset, const cpl_table *table,
                    const char *category, cpl_propertylist *header,
+				   cpl_propertylist * ext_header,
                    const cpl_parameterlist *parlist, const char *recipename, 
                    const char *version)
 {
@@ -1017,7 +1060,7 @@ int dfs_save_table(cpl_frameset *frameset, const cpl_table *table,
      */
     
     
-    if (cpl_table_save(table, plist, NULL, filename, CPL_IO_DEFAULT)) {
+    if (cpl_table_save(table, plist, ext_header, filename, CPL_IO_DEFAULT)) {
         cpl_msg_error(cpl_error_get_where(), "%s", cpl_error_get_message());
         cpl_msg_error(func, "Cannot save product %s to disk", filename);
         if (header == NULL)
@@ -1168,6 +1211,39 @@ int dfs_equal_keyword(cpl_frameset *frameset, const char *keyword)
 
 }
 
+cpl_error_code dfs_save_table_ext_name(
+		  cpl_table 		   * table,
+		  const char             * tag,
+		  cpl_propertylist * extheader,
+		  const char 	   *   extname){
+
+	cpl_propertylist * l = cpl_propertylist_new();
+	if(extheader){
+		cpl_propertylist_append(l, extheader);
+	}
+
+	cpl_propertylist_update_string(l, "EXTNAME", extname);
+	const cpl_error_code err = dfs_save_table_ext(table, tag, l);
+	cpl_propertylist_delete(l);
+
+	return err;
+}
+
+cpl_error_code dfs_save_image_ext_name(cpl_image * img, const char * tag,
+				  cpl_propertylist * extheader, const char * extname){
+
+	cpl_propertylist * l = cpl_propertylist_new();
+	if(extheader){
+		cpl_propertylist_append(l, extheader);
+	}
+
+	cpl_propertylist_update_string(l, "EXTNAME", extname);
+	const cpl_error_code err = dfs_save_image_ext(img, tag, l);
+	cpl_propertylist_delete(l);
+
+	return err;
+}
+
 /**
  * @brief Save a table in a extension (different from the first one)
  *
