@@ -46,6 +46,9 @@
 #include "vimos_response.h"
 #include "vimos_detected_slits.h"
 #include "vimos_flat_normalise.h"
+#include "vimos_spec_idp.h"
+
+#include "slit_trace_distortion.h"
 
 static int vimos_science_response_fill_ignore
 (const cpl_table * flux_table, const cpl_table * telluric_table,
@@ -68,6 +71,10 @@ static void vimos_shift_stdstar(cpl_image * stdstar_spectra,
                                 double dispersion, 
                                 double resp_shift);
 
+static bool
+convert_to_xy(const cpl_wcs * wcs, const double x, const double y,
+        double &ra, double &dec);
+
 #define vimos_science_exit(message)           \
 {                                             \
 if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
@@ -79,7 +86,6 @@ cpl_image_delete(mapped_cleaned);             \
 cpl_image_delete(skylocalmap);                \
 cpl_image_delete(skymap);                     \
 cpl_image_delete(smapped);                    \
-cpl_table_delete(offsets);                    \
 cpl_table_delete(sky);                        \
 if(master_bias != NULL) cpl_image_delete(master_bias);              \
 if(master_bias_err != NULL) cpl_image_delete(master_bias_err);      \
@@ -184,7 +190,6 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
     cpl_table        *slits          = NULL;
     cpl_table        *maskslits      = NULL;
     cpl_table        *polytraces     = NULL;
-    cpl_table        *offsets        = NULL;
     cpl_table        *sky            = NULL;
     cpl_table        *photcal        = NULL;
     cpl_table        *response_interp = NULL;
@@ -628,7 +633,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
 
     //TODO: it looks like the code after the if is never executed, since the 
     //sof always comes with one single science. The case of more science is 
-    //treated specially by the vimos_science_mult() function in vimos_science.cc
+    //treated specially by the vimos_science_internal() function in vimos_science.cc
     //which in turn calls this function but then only for each individual frame.
     //The problem is that there is then a lot of code duplication.
     //Note that the average in vimos_science.cc takes into account the
@@ -871,6 +876,9 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
     std::string grism_name = 
             cpl_propertylist_get_string(header, grism_name_key.str().c_str());
 
+    const int binning_y = cpl_propertylist_get_int(header, "ESO DET WIN1 BINY");
+    const double ccd_pix_size = cpl_propertylist_get_double(header, "ESO DET CHIP1 PSZX");
+
     if (standard) {
         if (!anyframe) {
 
@@ -954,36 +962,6 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
      * commented out.
      */
 
-//    maskslits = mos_load_slits_vimos(header);
-//
-//    /*
-//     * Check if all slits have the same X offset: in such case,
-//     * treat the observation as a long-slit one!
-//     */
-//
-//    mxpos = cpl_table_get_column_median(maskslits, "ytop");
-//    xpos = cpl_table_get_data_double(maskslits, "ytop");
-//    nslits = cpl_table_get_nrow(maskslits);
-//     
-//    treat_as_lss = 1;
-//    for (i = 0; i < nslits; i++) { 
-//        if (fabs(mxpos-xpos[i]) > 0.01) {
-//            treat_as_lss = 0;
-//            break;
-//        }
-//    }
-//
-//    treat_as_lss = 0;  // FIXME Prevent LSS treatment for the moment!!!
-//
-//    cpl_table_delete(maskslits); maskslits = NULL;
-//
-//    if (treat_as_lss) {
-//        cpl_msg_warning(recipe, "All MOS slits have the same offset: %.2f\n"
-//                        "The LSS data reduction strategy is applied!",
-//                        mxpos);
-//        skylines_offsets_tag   = "MOS_SKYLINES_OFFSETS_LONG";
-//    }
-
     if (treat_as_lss) {
         if (skylocal) {
             if (cosmics)
@@ -1217,6 +1195,8 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
     else
         highres = 1;
 
+    double sky_align_shift = NAN;
+
     if (skyalign >= 0) {
         if (skyalign) {
             cpl_msg_info(recipe, "Align wavelength solution to reference "
@@ -1227,6 +1207,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
             "skylines applying median offset...");
         }
 
+        cpl_table * offsets = NULL;
         if (treat_as_lss) {
             offsets = mos_wavelength_align_lss(smapped, reference, 
                                                startwavelength, endwavelength, 
@@ -1246,8 +1227,11 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
                                 "to reference sky lines may be unreliable in "
                                 "this case!");
 
-            if (dfs_save_table(frameset, offsets, skylines_offsets_tag, NULL, 
-                               parlist, recipe, version))
+            sky_align_shift = vimos_spec_idp_get_sky_align_shift(offsets, slits);
+
+
+            if (dfs_save_table_with_header_ext(frameset, offsets, skylines_offsets_tag, NULL,
+            		NULL, parlist, recipe, version))
                 vimos_science_exit(NULL);
 
             cpl_table_delete(offsets); offsets = NULL;
@@ -1257,7 +1241,13 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
                             "to reference sky lines could not be done!");
             skyalign = -1;
         }
-
+    }
+    else
+    {
+        cpl_table * offsets = mos_wavelength_align_get_offsets(startwavelength, endwavelength,
+                reference, 4, lines, highres, idscoeff, slits, smapped);
+        sky_align_shift = vimos_spec_idp_get_sky_align_shift(offsets, slits);
+        cpl_table_delete(offsets);
     }
 
     if (treat_as_lss) {
@@ -1325,7 +1315,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
             vimos_science_exit(NULL);
         cpl_image * mapped_sky_err =cpl_image_power_create(mapped_sky_var, 0.5);
         cpl_image_divide_scalar(mapped_sky_err, alltime);
-        if (dfs_save_image_ext(mapped_sky_err, mapped_science_sky_tag, header))
+        if (dfs_save_image_ext_name(mapped_sky_err, mapped_science_sky_tag, header, "ERROR"))
             vimos_science_exit("Cannot save error extension");
         cpl_image_delete(mapped_sky_err);
         cpl_image_delete(dummy); dummy = NULL;
@@ -1335,7 +1325,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
                            header, parlist, recipe, version))
             vimos_science_exit(NULL);
         cpl_image * mapped_sky_err =cpl_image_power_create(mapped_sky_var, 0.5);
-        if (dfs_save_image_ext(mapped_sky_err, mapped_science_sky_tag, header))
+        if (dfs_save_image_ext_name(mapped_sky_err, mapped_science_sky_tag, header, "ERROR"))
             vimos_science_exit("Cannot save error extension");
         cpl_image_delete(mapped_sky_err);
     }
@@ -1594,7 +1584,22 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
 
         cpl_image_delete(dummy); dummy = NULL;
 
+        cpl_propertylist * science_header = dfs_load_header(frameset, science_tag, 0);
+        vimos_preoverscan ps_scan;
+        mosca::rect_region overscan_region = ps_scan.get_wcs_crop_region(science_header);
+        cpl_wcs *wcs = cpl_wcs_new_from_propertylist(science_header);
+
+        const double undeviated_wlen = get_undeviated_wlen(science_header);
+        vimos_science_set_object_coord(slits,
+					wave_cal, polytraces, wcs,
+					overscan_region,
+					undeviated_wlen, ny, science_header);
+
+        cpl_propertylist_delete(science_header);
+        cpl_wcs_delete(wcs);
+
         mos_rotate_slits(slits, rotate, ny, nx);
+
         if (dfs_save_table(frameset, slits, object_table_tag, NULL, parlist, 
                            recipe, version))
             vimos_science_exit(NULL);
@@ -1691,7 +1696,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
 
                     dummy = cpl_image_new(wcount, 1, CPL_TYPE_FLOAT);
                     data = cpl_image_get_data_float(dummy);
-                    map_table(dummy, wstart, wstep, photcal,
+                    map_table(dummy, wstart, wstep, response_interp,
                               "WAVE", "EFFICIENCY");
 
                     for (i = 0; i < wcount; i++) {
@@ -1811,6 +1816,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
                 cpl_image_delete(science_images); science_images = NULL;
             }
 
+            cpl_propertylist_update_double(header, "ESO PRO EXPTTOT", alltime);
             if (time_normalise) {
                 cpl_propertylist_update_string(header, "BUNIT", "ADU/s");
                 cpl_image_divide_scalar(images[0], alltime);
@@ -1924,13 +1930,18 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
         }
     }
 
-
-    if (skyalign >= 0) {
-        if (dfs_save_table(frameset, idscoeff, disp_coeff_sky_tag, NULL, 
-                           parlist, recipe, version))
-            vimos_science_exit(NULL);
+    cpl_propertylist * plist = NULL;
+    if(!std::isnan(sky_align_shift)){
+        plist = cpl_propertylist_new();
+        cpl_propertylist_update_double(plist, idp_input_screen_skyaling_shift, sky_align_shift);
     }
 
+	if (dfs_save_table_with_header_ext(frameset, idscoeff, disp_coeff_sky_tag, NULL,
+	        plist, parlist, recipe, version)){
+		vimos_science_exit(NULL);
+	}
+
+	cpl_propertylist_delete(plist);
     cpl_table_delete(idscoeff); idscoeff = NULL;
 
     if (skyglobal || skymedian || skylocal) {
@@ -2000,7 +2011,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
         cpl_image * mapped_err = cpl_image_power_create(mapped_var, 0.5);
         if (time_normalise) 
             cpl_image_divide_scalar(mapped_err, alltime);
-        if (dfs_save_image_ext(mapped_err, mapped_science_tag, header))
+        if (dfs_save_image_ext_name(mapped_err, mapped_science_tag, header, "ERROR"))
             vimos_science_exit("Cannot save error extension");
         cpl_image_delete(mapped_err);
     }
@@ -2295,4 +2306,224 @@ static void vimos_shift_stdstar(cpl_image * stdstar_spectra,
     cpl_image_shift(stdstar_spectra, shift_pix, 0);
     cpl_image_shift(stdstar_spectra_error, shift_pix, 0);
 }
+
+
+static
+double get_undeviated_wlen_from_name_and_mjd(std::string name, const double mjd_obs){
+
+	auto safe_to_lower = [](unsigned char c){ return std::tolower(c); };
+	std::transform(name.cbegin(), name.cend(), name.begin(),  safe_to_lower);
+
+
+	if(name == "lr_red") return 7500.0;
+	if(name == "lr_blue") return 4800.0;
+
+	if(name == "mr") return 7000.0;
+
+	if(name == "hr_orange") return 6310.0;
+
+	if(name == "hr_red"){
+		if(mjd_obs > 56209.0) //2012-10-09
+			return 7500.0;
+		else
+			return NAN;
+	}
+
+	if(name == "hr_blue"){
+
+		if(mjd_obs < 56001.0) //2012-03-15
+			return 5100.0;
+		else
+			return 4020.0;
+
+	}
+
+	return NAN;
+}
+
+static
+std::string propertylist_get_std_string(const cpl_propertylist * l, const char * tag){
+
+	const char * s = cpl_propertylist_get_string(l, tag);
+	if(s == NULL){
+		return "";
+	}
+
+	return s;
+}
+
+double get_undeviated_wlen(const cpl_propertylist * science_header){
+
+    if(cpl_error_get_code()) return NAN;
+
+    const char * grism_name_tag = get_grism_name_tag(science_header);
+
+    if(!grism_name_tag){
+        return NAN;
+    }
+
+    std::string grism_name = propertylist_get_std_string(science_header, grism_name_tag);
+
+    if(grism_name.empty() || cpl_error_get_code()){
+        cpl_error_reset();
+        return NAN;
+    }
+
+    const double mjd_obs = cpl_propertylist_get_double(science_header, "MJD-OBS");
+    if(cpl_error_get_code()){
+        cpl_error_reset();
+        return NAN;
+    }
+
+    const double to_ret = get_undeviated_wlen_from_name_and_mjd(grism_name, mjd_obs);
+    cpl_msg_info(cpl_func, "For GRISM %s found undeviated wavelength %f", grism_name.c_str(), to_ret);
+    return to_ret;
+}
+
+bool is_posang_valid(const cpl_propertylist * plist, double & posang){
+
+    posang = cpl_propertylist_get_double(plist, "ESO ADA POSANG");
+
+    if(cpl_error_get_code()){
+        cpl_error_reset();
+        posang = -1;
+        return false;
+    }
+
+    return posang == 0.0 || posang == 90.0;
+}
+
+#define MAX_COLNAME      (80)
+void vimos_science_set_object_coord(cpl_table * objects,
+				mosca::wavelength_calibration& wave_cal,
+				cpl_table * polytraces, const cpl_wcs *wcs,
+				const mosca::rect_region& overscan_region,
+				const double undeviated_wlen,
+				const double nx,
+				const cpl_propertylist * science_header)
+{
+    int nobjects = 0;
+
+    char        name[MAX_COLNAME];
+    cpl_size maxobjects = 1;
+    snprintf(name, MAX_COLNAME, "object_%" CPL_SIZE_FORMAT, maxobjects);
+    while (cpl_table_has_column(objects, name)) {
+        maxobjects++;
+        snprintf(name, MAX_COLNAME, "object_%" CPL_SIZE_FORMAT, maxobjects);
+    }
+
+    cpl_size nslits = cpl_table_get_nrow(objects);
+
+    for (cpl_size i = 0; i < nslits; i++) {
+        for (cpl_size j = 1; j < maxobjects; j++) {
+            snprintf(name, MAX_COLNAME, "object_%" CPL_SIZE_FORMAT, j);
+            if (cpl_table_is_valid(objects, name, i))
+                nobjects++;
+        }
+    }
+
+
+    for (cpl_size i = 0; i < nslits; i++) {
+        for (cpl_size j = 1; j < maxobjects; j++) {
+
+            snprintf(name, MAX_COLNAME, "ra_%" CPL_SIZE_FORMAT, j);
+            if (!cpl_table_has_column(objects, name))
+            {
+                cpl_table_new_column(objects, name, CPL_TYPE_DOUBLE);
+                cpl_table_set_column_unit(objects, name, "deg");
+            }
+
+            snprintf(name, MAX_COLNAME, "dec_%" CPL_SIZE_FORMAT, j);
+            if (!cpl_table_has_column(objects, name))
+            {
+                cpl_table_new_column(objects, name, CPL_TYPE_DOUBLE);
+                cpl_table_set_column_unit(objects, name, "deg");
+            }
+
+        }
+    }
+    /* RA/DEC must always be added otherwise the interactive windows
+     * in the reflex workflow won't work, hence we test the validity conditions
+     * for RA/DEC calculation here*/
+    if(std::isnan(undeviated_wlen)){
+        cpl_msg_warning(cpl_func, "Unable to determine Undeviated Wavelength");
+        return;
+    }
+
+    double posang = 0;
+    if(!is_posang_valid(science_header, posang)){
+        cpl_msg_warning(cpl_func, "ADA.POSANG %f is not allowed in VIMOS MOS, "
+                "object RA/DEC calculation is disabled", posang);
+        return;
+    }
+
+    mosca::slit_trace_distortion spa_dist(polytraces);
+    nobjects = 0;
+    for (cpl_size i = 0; i < nslits; i++) {
+        for (cpl_size j = 1; j < maxobjects; j++) {
+            snprintf(name, MAX_COLNAME, "object_%" CPL_SIZE_FORMAT, j);
+            if (cpl_table_is_valid(objects, name, i)) {
+                snprintf(name, MAX_COLNAME, "start_%" CPL_SIZE_FORMAT, j);
+                const int obj_low = cpl_table_get_int(objects, name, i, NULL);
+                snprintf(name, MAX_COLNAME, "end_%" CPL_SIZE_FORMAT, j);
+                const int obj_high = cpl_table_get_int(objects, name, i, NULL);
+                const double obj_center_spa = (obj_low + obj_high) / 2.;
+                double obj_center_disp =
+                         wave_cal.get_pixel(obj_center_spa, undeviated_wlen);
+
+                double obj_center_spa_noncorr;
+                bool success = spa_dist.to_distorted(obj_center_spa,
+                        obj_center_disp, obj_center_spa_noncorr, objects);
+
+                if(!success)
+                    continue;
+
+                obj_center_spa_noncorr = nx - obj_center_spa_noncorr + overscan_region.llx();
+                obj_center_disp += overscan_region.lly();
+
+                double ra, dec;
+                /* Convert to Ra, Dec from X,Y  */
+                success = convert_to_xy(wcs, obj_center_spa_noncorr, obj_center_disp, ra, dec);
+
+                if(!success)
+                    continue;
+
+                snprintf(name, MAX_COLNAME, "ra_%" CPL_SIZE_FORMAT, j);
+                cpl_table_set_double(objects, name, i, ra);
+
+                snprintf(name, MAX_COLNAME, "dec_%" CPL_SIZE_FORMAT, j);
+                cpl_table_set_double(objects, name, i, dec);
+            }
+        }
+    }
+}
+
+static bool
+convert_to_xy(const cpl_wcs * wcs, const double x, const double y, double &ra, double &dec){
+
+    cpl_matrix *    from_coord = cpl_matrix_new(1, 2);
+
+    cpl_matrix_set(from_coord, 0, 0, x);
+    cpl_matrix_set(from_coord, 0, 1, y);
+
+    cpl_matrix *    to_coord = NULL;
+    cpl_array *     status = NULL;
+
+    bool success = true;
+    if(cpl_wcs_convert(wcs, from_coord, &to_coord,
+                       &status, CPL_WCS_PHYS2WORLD)){
+        cpl_error_reset();
+        success = false;
+    }
+    else{
+        ra  = cpl_matrix_get(to_coord, 0, 0);
+        dec = cpl_matrix_get(to_coord, 0, 1);
+    }
+
+    cpl_array_delete(status);
+    cpl_matrix_delete(from_coord);
+    cpl_matrix_delete(to_coord);
+    return success;
+}
+
 /**@}*/
