@@ -30,10 +30,26 @@
 
 #include <cpl.h>
 
+#include <vector>
+#include "wavelength_calibration.h"
+#include "rect_region.h"
+
 CPL_BEGIN_DECLS
 
 int vimos_science_impl(cpl_frameset *, cpl_parameterlist *, int);
 
+void vimos_science_set_object_coord(cpl_table * objects,
+								 mosca::wavelength_calibration& wave_cal,
+								 cpl_table* polytraces, const cpl_wcs *wcs,
+								 const mosca::rect_region& overscan_region,
+								 const double undeviated_wlen,
+								 const double nx,
+								 const cpl_propertylist * science_header);
+
+double get_undeviated_wlen(const cpl_propertylist * science_header);
+
+bool is_posang_valid(const cpl_propertylist * plist, double & posang);
+
 CPL_END_DECLS
 
 #endif
