@@ -46,6 +46,7 @@
 #include "vmtable.h"
 #include "vmdistmodels.h"
 #include "vmadf.h"
+#include "cpl.h"
 
 
 /* 
@@ -58,7 +59,7 @@ VimosBezierCurve *newBezierCurve()
   VimosBezierCurve *tmpCurve;
   
   /* allocate space */
-  tmpCurve = (VimosBezierCurve *) pil_malloc(sizeof(VimosBezierCurve));
+  tmpCurve = (VimosBezierCurve *) cpl_malloc(sizeof(VimosBezierCurve));
   
   /* check if space was allocated */
   if (tmpCurve == NULL) {
@@ -86,7 +87,7 @@ void deleteBezierCurve(VimosBezierCurve *curve)
 {
   /* only have to free the space */
   if (curve != NULL) {
-    pil_free(curve);
+    cpl_free(curve);
   }
   
 }
@@ -133,7 +134,7 @@ VimosAdfRectSlit *newAdfRectSlit()
   VimosAdfRectSlit *tmpSlit;
   
   /* allocate space */
-  tmpSlit = (VimosAdfRectSlit *) pil_malloc(sizeof(VimosAdfRectSlit));
+  tmpSlit = (VimosAdfRectSlit *) cpl_malloc(sizeof(VimosAdfRectSlit));
   
   /* check if space was allocated */
   if (tmpSlit == NULL) {
@@ -162,7 +163,7 @@ void deleteAdfRectSlit(VimosAdfRectSlit *adfSlit)
 {
   /* only have to free the space */
   if (adfSlit != NULL) {
-    pil_free(adfSlit);
+    cpl_free(adfSlit);
   }
   
 }
@@ -178,7 +179,7 @@ VimosAdfCurvSlit *newAdfCurvSlit()
   VimosAdfCurvSlit *tmpSlit;
   
   /* allocate space */
-  tmpSlit = (VimosAdfCurvSlit *) pil_malloc(sizeof(VimosAdfCurvSlit));
+  tmpSlit = (VimosAdfCurvSlit *) cpl_malloc(sizeof(VimosAdfCurvSlit));
   
   /* check if space was allocated */
   if (tmpSlit == NULL) {
@@ -195,7 +196,7 @@ VimosAdfCurvSlit *newAdfCurvSlit()
   /* middle X curve */
   tmpSlit->xMiddle = newBezierCurve();
   if (tmpSlit->xMiddle == NULL) {
-    pil_free(tmpSlit);
+    cpl_free(tmpSlit);
     cpl_msg_error(modName, "The function newBezierCurve has returned NULL (x)");
     return(NULL);
   }
@@ -203,7 +204,7 @@ VimosAdfCurvSlit *newAdfCurvSlit()
   /* middle Y curve */
   tmpSlit->yMiddle = newBezierCurve();
   if (tmpSlit->yMiddle == NULL) {
-    pil_free(tmpSlit);
+    cpl_free(tmpSlit);
     cpl_msg_error(modName, "The function newBezierCurve has returned NULL (y)");
     return(NULL);
   }
@@ -225,7 +226,7 @@ void deleteAdfCurvSlit(VimosAdfCurvSlit *slit)
     deleteBezierCurve(slit->yMiddle);
     
     /* free the object */
-    pil_free(slit);
+    cpl_free(slit);
   }
   
 }
@@ -239,7 +240,7 @@ VimosAdfCircSlit *newAdfCircSlit()
   VimosAdfCircSlit *tmpSlit;
   
   /* allocate space */
-  tmpSlit = (VimosAdfCircSlit *) pil_malloc(sizeof(VimosAdfCircSlit));
+  tmpSlit = (VimosAdfCircSlit *) cpl_malloc(sizeof(VimosAdfCircSlit));
   
   /* check if space was allocated */
   if (tmpSlit == NULL) {
@@ -270,7 +271,7 @@ void deleteAdfCircSlit(VimosAdfCircSlit *adfSlit)
 {
   /* only have to free the space */
   if (adfSlit != NULL) {
-    pil_free(adfSlit);
+    cpl_free(adfSlit);
   }
   
 }
@@ -286,7 +287,7 @@ VimosAdfRefrSlit *newAdfRefrSlit()
   VimosAdfRefrSlit *tmpSlit;
   
   /* allocate space */
-  tmpSlit = (VimosAdfRefrSlit *) pil_malloc(sizeof(VimosAdfRefrSlit));
+  tmpSlit = (VimosAdfRefrSlit *) cpl_malloc(sizeof(VimosAdfRefrSlit));
   
   /* check if space was allocated */
   if (tmpSlit == NULL) {
@@ -315,7 +316,7 @@ void deleteAdfRefrSlit(VimosAdfRefrSlit *adfSlit)
 
   /* only have to free the space */
   if (adfSlit != NULL) {
-    pil_free(adfSlit);
+    cpl_free(adfSlit);
   }
   
 }
@@ -326,7 +327,7 @@ VimosAdfSlitHolder *newAdfSlitHolder()
   const char modName[] = "newAdfSlitHolder";
   VimosAdfSlitHolder *slitHolder;
   
-  slitHolder = (VimosAdfSlitHolder *) pil_malloc(sizeof(VimosAdfSlitHolder));
+  slitHolder = (VimosAdfSlitHolder *) cpl_malloc(sizeof(VimosAdfSlitHolder));
   
   if (slitHolder == NULL) {
     cpl_msg_error(modName, "Allocation Error");
@@ -379,7 +380,7 @@ void deleteAdfSlitHolder(VimosAdfSlitHolder *slitHolder)
       }
     }
     nextSlit = tSlit->next;
-    pil_free(tSlit);
+    cpl_free(tSlit);
     tSlit = nextSlit;
   }
 
@@ -1044,14 +1045,14 @@ VimosBool calcSlitLocationsOnCCD(void *slit, VimosAdfSlitType slitType,
         return(VM_FALSE);
       }
 
-      *crvPol=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *crvPol=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*crvPol == NULL) {
         cpl_msg_error(modName, "Allocation Error");
         return(VM_FALSE);
       }
 
-      *invDis=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *invDis=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*invDis == NULL) {
         cpl_msg_error(modName, "Allocation Error");
@@ -1226,14 +1227,14 @@ VimosBool calcSlitLocationsOnCCD(void *slit, VimosAdfSlitType slitType,
         return(VM_FALSE);
       }
 
-      *crvPol=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *crvPol=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*crvPol == NULL) {
         cpl_msg_error(modName, "Allocation Error");
         return(VM_FALSE);
       }
 
-      *invDis=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *invDis=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*invDis == NULL) {
         cpl_msg_error(modName, "Allocation Error");
@@ -1430,13 +1431,13 @@ VimosBool calcSlitLocationsOnCCD(void *slit, VimosAdfSlitType slitType,
         return(VM_FALSE);
       }
 
-      *crvPol=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *crvPol=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*crvPol == NULL) {
         cpl_msg_error(modName, "Allocation Error");
         return(VM_FALSE);
       }
-      *invDis=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *invDis=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*invDis == NULL) {
         cpl_msg_error(modName, "Allocation Error");
@@ -1600,14 +1601,14 @@ VimosBool calcSlitLocationsOnCCD(void *slit, VimosAdfSlitType slitType,
         return(VM_FALSE);
       }
 
-      *crvPol=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *crvPol=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*crvPol == NULL) {
         cpl_msg_error(modName, "Allocation Error");
         return(VM_FALSE);
       }
 
-      *invDis=(VimosDistModel1D **)pil_malloc(*numPix*sizeof(VimosDistModel1D *));
+      *invDis=(VimosDistModel1D **)cpl_malloc(*numPix*sizeof(VimosDistModel1D *));
       /* check if space was allocated */
       if (*invDis == NULL) {
         cpl_msg_error(modName, "Allocation Error");
