@@ -38,6 +38,7 @@
 #include <cpl_msg.h>
 
 #include "vmmath.h"
+#include "cpl.h"
 
 
 /* private functions */
@@ -218,7 +219,7 @@ static double *generateInterpolationKernel(char *kernel_type)
   } 
 
   if (!strcmp(kernel_type, "sinc")) {
-    tab = (double*) pil_malloc(samples * sizeof(double)) ;
+    tab = (double*) cpl_malloc(samples * sizeof(double)) ;
 
   /* check if space was allocated */
     if (tab == NULL) {
@@ -236,7 +237,7 @@ static double *generateInterpolationKernel(char *kernel_type)
   }  
   
   if (!strcmp(kernel_type, "sinc2")) {
-    tab = (double*) pil_malloc(samples * sizeof(double)) ;
+    tab = (double*) cpl_malloc(samples * sizeof(double)) ;
 
   /* check if space was allocated */
     if (tab == NULL) {
@@ -255,7 +256,7 @@ static double *generateInterpolationKernel(char *kernel_type)
   }  
   
   if (!strcmp(kernel_type, "lanczos")) {
-    tab = (double*) pil_malloc(samples * sizeof(double)) ;
+    tab = (double*) cpl_malloc(samples * sizeof(double)) ;
 
   /* check if space was allocated */
     if (tab == NULL) {
@@ -275,7 +276,7 @@ static double *generateInterpolationKernel(char *kernel_type)
   } 
 
   if (!strcmp(kernel_type, "hamming")) {
-    tab = (double*) pil_malloc(samples * sizeof(double)) ;
+    tab = (double*) cpl_malloc(samples * sizeof(double)) ;
 
   /* check if space was allocated */
     if (tab == NULL) {
@@ -297,7 +298,7 @@ static double *generateInterpolationKernel(char *kernel_type)
   } 
 
   if (!strcmp(kernel_type, "hann")) {
-    tab = (double*) pil_malloc(samples * sizeof(double)) ;
+    tab = (double*) cpl_malloc(samples * sizeof(double)) ;
 
   /* check if space was allocated */
     if (tab == NULL) {
@@ -389,7 +390,7 @@ static double *generateTanhKernel(double steep)
    * Generate the kernel expression in Fourier space
    * with a correct frequency ordering to allow standard FT
    */
-  x = (double*) pil_malloc((2*np+1)*sizeof(double)) ;
+  x = (double*) cpl_malloc((2*np+1)*sizeof(double)) ;
 
   /* check if space was allocated */
     if (x == NULL) {
@@ -416,7 +417,7 @@ static double *generateTanhKernel(double steep)
   /*
    * Allocate and fill in returned array
    */
-  kernel = (double*) pil_malloc(samples * sizeof(double)) ;
+  kernel = (double*) cpl_malloc(samples * sizeof(double)) ;
 
   /* check if space was allocated */
     if (kernel == NULL) {
@@ -427,7 +428,7 @@ static double *generateTanhKernel(double steep)
   for (i = 0 ; i < samples ; i++) {
     kernel[i] = 2.0 * width * x[2*i] * inv_np ;
   }
-  pil_free(x) ;
+  cpl_free(x) ;
   return kernel ;
 }
 
@@ -511,7 +512,7 @@ VimosBool setupInterpolation(double **kernel, VimosLong32 **leaps,
     return(VM_FALSE);
   }
   
-  tmpLong = (VimosLong32 *) pil_malloc(16*sizeof(VimosLong32));
+  tmpLong = (VimosLong32 *) cpl_malloc(16*sizeof(VimosLong32));
 
   /* check if space was allocated */
     if (tmpLong == NULL) {
@@ -671,6 +672,34 @@ double computeVarianceFloat2D(float a[], int nx, int ny)
   
 }
 
+double computeVarianceDouble2D(double a[], int nx, int ny)
+{
+
+  int    x, y;
+  int    n = 0;
+  int    dx = 1;
+  int    dy = 1;
+  double delta;
+  double variance = 0.0;
+
+  if (nx < 4 || ny < 4)
+    return 0;
+
+  nx--;
+  ny--;
+
+  for (x = 0; x < nx; x++) {
+    for (y = 0; y < ny; y++) {
+      delta = a[x + y * nx] - a[x + dx + (y + dy) * nx];
+      variance = (((double)n)/(n+1))*variance + ((double)delta*delta)/(n+1);
+      n++;
+    }
+  }
+
+  return variance / 2.0;
+  
+}
+
 
 /*************************************************************************/
 
@@ -682,7 +711,7 @@ float *extractFloatImage
   char   modName[] = "extractFloatImage";
 
   if (x >= 0 && y >= 0 && sizex >= x + nx && sizey >= y + ny) {
-    extractedImage = (float *) pil_malloc(nx*ny*sizeof(float));
+    extractedImage = (float *) cpl_malloc(nx*ny*sizeof(float));
     for (i=0; i<ny; i++) {
       for (j=0; j<nx; j++) {
         extractedImage[nx*i+j] = a[sizex*(y+i)+x+j];
@@ -739,7 +768,7 @@ VimosDpoint *darrayHistogram(double *darray, unsigned int arDim,
   }
 
   bin_size = (double)(max - min)/(double)nbins;
-  h = (unsigned int*)pil_calloc(nbins,sizeof(unsigned int)) ;
+  h = (unsigned int*)cpl_calloc(nbins,sizeof(unsigned int)) ;
  
   for (i=0 ; i<arDim ; i++) { 
     if (darray[i] >= max) {
@@ -755,7 +784,7 @@ VimosDpoint *darrayHistogram(double *darray, unsigned int arDim,
      histogram[i].x = min + (double)i * bin_size ;
      histogram[i].y = (double)h[i] ;
   }
-  pil_free(h) ;
+  cpl_free(h) ;
   return histogram ;
 }
 
@@ -774,7 +803,7 @@ VimosDpoint *darrayHistoStartEnd(double *darray, int arrDim, double start,
   }
 
   nbins = (floor)((start-end)/bin_size);
-  h = (unsigned int*)pil_calloc(nbins,sizeof(unsigned int)) ;
+  h = (unsigned int*)cpl_calloc(nbins,sizeof(unsigned int)) ;
  
   for (i=0 ; i<arrDim ; i++) { 
     if(((int)(darray[i] - start) > 0) && ((int)(end - darray[i]) > 0))
@@ -786,7 +815,7 @@ VimosDpoint *darrayHistoStartEnd(double *darray, int arrDim, double start,
     histogram[i].x = start + (double)i * bin_size ;
     histogram[i].y = (double)h[i] ;
   }
-  pil_free(h) ;
+  cpl_free(h) ;
   return histogram ;
 }
 
@@ -849,7 +878,7 @@ int  waterShed(float *profile, int numPoints, int numLevels,
   float sum, num;
 
   /* copy input profile and box smooth */
-  tmpProf = (float *) pil_malloc(numPoints*sizeof(float));
+  tmpProf = (float *) cpl_malloc(numPoints*sizeof(float));
 
   /* check if space was allocated */
     if (tmpProf == NULL) {
@@ -1023,7 +1052,7 @@ int  waterShed(float *profile, int numPoints, int numLevels,
   return(numObjects);
   
   
-  pil_free(tmpProf);
+  cpl_free(tmpProf);
   
 }
 
@@ -1123,51 +1152,99 @@ float opt_med9(float  *p)
 
 
 /*---------------------------------------------------------------------------
-   Function	:	medianPixelvalue()
-   In 		:	allocated array of pixelvalues, # of pixels in the array
-   Out 		:	1 pixel value
-   Job		:	compute the median pixel value out of an array
-   Notice	:	calls the fastest method depending on the number of
-				elements in input.
-			     
+   Function :   medianPixelvalue()
+   In       :   allocated array of pixelvalues, # of pixels in the array
+   Out      :   1 pixel value
+   Job      :   compute the median pixel value out of an array
+   Notice   :   calls the fastest method depending on the number of
+                elements in input.
+                 
  ---------------------------------------------------------------------------*/
 
 
 
 float medianPixelvalue(float * a, int n)
 {
-  float	 medianVal ;
+  float  medianVal ;
   float *a_copy;
   int    i;
 
   if(n==1) return a[0];
 
-  a_copy = (float*) pil_malloc (n*sizeof(float));
+  a_copy = (float*) cpl_malloc (n*sizeof(float));
   for (i=0; i<n; i++) a_copy[i] = a[i];
 
-	switch(n) {
-	case 3:
-	  medianVal = opt_med3(a_copy);
-	  break ;
+    switch(n) {
+    case 3:
+      medianVal = opt_med3(a_copy);
+      break ;
 
-	case 5:
-	  medianVal = opt_med5(a_copy);
-	  break ;
+    case 5:
+      medianVal = opt_med5(a_copy);
+      break ;
 
-	case 7:
-	  medianVal = opt_med7(a_copy);
-	  break ;
+    case 7:
+      medianVal = opt_med7(a_copy);
+      break ;
 
-	case 9:
-	  medianVal = opt_med9(a_copy);
-	  break ;
+    case 9:
+      medianVal = opt_med9(a_copy);
+      break ;
 
-	default:
-	  medianVal = ((n>1000) ? medianWirth(a_copy,n): median(a_copy,n));
-	  break ;
-	}
-	pil_free(a_copy);
-	return medianVal;
+    default:
+      medianVal = ((n>1000) ? medianWirth(a_copy,n): median(a_copy,n));
+      break ;
+    }
+    cpl_free(a_copy);
+    return medianVal;
+}
+
+/*---------------------------------------------------------------------------
+   Function :   medianPixelvalue()
+   In       :   allocated array of pixelvalues, # of pixels in the array
+   Out      :   1 pixel value
+   Job      :   compute the median pixel value out of an array
+   Notice   :   calls the fastest method depending on the number of
+                elements in input.
+                 
+ ---------------------------------------------------------------------------*/
+
+
+
+double medianPixelvalueDouble(double * a, int n)
+{
+  float  medianVal ;
+  float *a_copy;
+  int    i;
+
+  if(n==1) return a[0];
+
+  a_copy = (float*) cpl_malloc (n*sizeof(float));
+  for (i=0; i<n; i++) a_copy[i] = a[i];
+
+    switch(n) {
+    case 3:
+      medianVal = opt_med3(a_copy);
+      break ;
+
+    case 5:
+      medianVal = opt_med5(a_copy);
+      break ;
+
+    case 7:
+      medianVal = opt_med7(a_copy);
+      break ;
+
+    case 9:
+      medianVal = opt_med9(a_copy);
+      break ;
+
+    default:
+      medianVal = ((n>1000) ? medianWirth(a_copy,n): median(a_copy,n));
+      break ;
+    }
+    cpl_free(a_copy);
+    return medianVal;
 }
 
 VimosBool findPeak1D(float *data, int size, float *position, int minPoints) 
@@ -1183,10 +1260,10 @@ VimosBool findPeak1D(float *data, int size, float *position, int minPoints)
  /*
   *  Find median
   */
-  copy = (float *) pil_malloc(size*sizeof(float));
+  copy = (float *) cpl_malloc(size*sizeof(float));
   for (i=0; i<size; i++) copy[i] = data[i];
   median = medianWirth(copy, size);
-  pil_free(copy);
+  cpl_free(copy);
 
  /*
   *  Find max
@@ -1269,10 +1346,10 @@ VimosBool findPeak2D(float *data, int sizex, int sizey,
  /*
   *  Find median
   */
-  copy = (float *) pil_malloc(size*sizeof(float));
+  copy = (float *) cpl_malloc(size*sizeof(float));
   for (i=0; i<size; i++) copy[i] = data[i];
   median = medianWirth(copy, size);
-  pil_free(copy);
+  cpl_free(copy);
 
  /*
   *  Find max
@@ -1387,7 +1464,7 @@ float median(float x[],int n)
    float *xwork,medVal;
    int i,n2;
   
-   xwork = (float *) pil_malloc(n*sizeof(float));
+   xwork = (float *) cpl_malloc(n*sizeof(float));
   
    for (i = 0; i < n; i++)
      xwork[i] = x[i];
@@ -1399,7 +1476,7 @@ float median(float x[],int n)
    else
      medVal = xwork[n2];
 
-   pil_free(xwork);
+   cpl_free(xwork);
    return medVal;
 }
 
@@ -1456,11 +1533,11 @@ void xbiwt(float xdata[],int n,float *xlbiwt,float *xsbiwt,float *xlbiwt1,
    float *u1,*u2,*xlb,*xsb,xm,xmadm,c1,c2,xmm,s1,s2,s3,s4,*xwork;
    int i,j;
 
-   u1 = (float *) pil_malloc(n*sizeof(float));
-   u2 = (float *) pil_malloc(n*sizeof(float));
-   xlb = (float *) pil_malloc(11*sizeof(float));
-   xsb = (float *) pil_malloc(11*sizeof(float));
-   xwork = (float *) pil_malloc(n*sizeof(float));
+   u1 = (float *) cpl_malloc(n*sizeof(float));
+   u2 = (float *) cpl_malloc(n*sizeof(float));
+   xlb = (float *) cpl_malloc(11*sizeof(float));
+   xsb = (float *) cpl_malloc(11*sizeof(float));
+   xwork = (float *) cpl_malloc(n*sizeof(float));
   
    for (i = 0; i < n; i++)
      xwork[i] = xdata[i];
@@ -1560,11 +1637,11 @@ void xbiwt(float xdata[],int n,float *xlbiwt,float *xsbiwt,float *xlbiwt1,
 
    *xlbiwt = xlb[10];
    *xsbiwt = xsb[10];
-   pil_free(xwork);
-   pil_free(u1);
-   pil_free(u2);
-   pil_free(xlb);
-   pil_free(xsb);
+   cpl_free(xwork);
+   cpl_free(u1);
+   cpl_free(u2);
+   cpl_free(xlb);
+   cpl_free(xsb);
 }
 
 
@@ -1594,13 +1671,13 @@ float xmad(float xdata[],int n,float xmed)
    float *xdata2,madVal;
    int i;
 
-   xdata2 = (float *) pil_malloc(n*sizeof(float));
+   xdata2 = (float *) cpl_malloc(n*sizeof(float));
 
    for(i = 0; i < n; i++)
         xdata2[i] = fabs(xdata[i] - xmed);
         
    madVal = median(xdata2,n); 
-   pil_free(xdata2);
+   cpl_free(xdata2);
    return madVal;
 }
 
@@ -1616,7 +1693,7 @@ float *vector(long nl, long nh)
 {
         float *v;
  
-        v=(float *)pil_malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
+        v=(float *)cpl_malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
         return v-nl+NR_END;
 }
 
@@ -1624,7 +1701,7 @@ void free_vector(float *v, long nl, long nh)
 /* free a float vector allocated with vector() */
 {
   nh = nh ;
-        pil_free((FREE_ARG) (v+nl-NR_END));
+        cpl_free((FREE_ARG) (v+nl-NR_END));
 }
 
 
@@ -1636,10 +1713,10 @@ VimosBool findDip1D(float *data, int size, float *position, int minPoints)
   float    *neg;
   VimosBool status;
 
-  neg = (float *) pil_malloc(size*sizeof(float));
+  neg = (float *) cpl_malloc(size*sizeof(float));
   for (i=0; i<size; i++) neg[i] = -data[i];
   status = findPeak1D(neg, size, position, minPoints);
-  pil_free(neg);
+  cpl_free(neg);
   return status;
 }
 
@@ -1651,10 +1728,10 @@ VimosBool findUpJump(float *data, int size, float *position, int minPoints)
   VimosBool status;
 
   derSize = size - 1;
-  der = (float *) pil_malloc(derSize*sizeof(float));
+  der = (float *) cpl_malloc(derSize*sizeof(float));
   for (i=0; i<derSize; i++) der[i] = MAX(data[i+1]-data[i],0.);
   status = findPeak1D(der, derSize, position, minPoints);
-  pil_free(der);
+  cpl_free(der);
   if (status == VM_TRUE) *position += 0.5;
   return status;
 }
@@ -1667,10 +1744,10 @@ VimosBool findJump(float *data, int size, float *position, int minPoints)
   VimosBool status;
 
   derSize = size - 1;
-  der = (float *) pil_malloc(derSize*sizeof(float));
+  der = (float *) cpl_malloc(derSize*sizeof(float));
   for (i=0; i<derSize; i++) der[i] = fabs(data[i+1]-data[i]);
   status = findPeak1D(der, derSize, position, minPoints);
-  pil_free(der);
+  cpl_free(der);
   if (status == VM_TRUE) *position += 0.5;
   return status;
 }
@@ -1681,10 +1758,10 @@ VimosBool findDownJump(float *data, int size, float *position, int minPoints)
   float    *neg;
   VimosBool status;
 
-  neg = (float *) pil_malloc(size*sizeof(float));
+  neg = (float *) cpl_malloc(size*sizeof(float));
   for (i=0; i<size; i++) neg[i] = -data[i];
   status = findUpJump(neg, size, position, minPoints);
-  pil_free(neg);
+  cpl_free(neg);
   return status;
 }
 
