@@ -47,6 +47,7 @@
 #include "vmmath.h"
 #include "vmfit.h"
 #include "vmmosflat.h"
+#include "cpl.h"
 
 
 #define EXTRA              20.
@@ -211,11 +212,11 @@ VmSpNormPoly(VimosImage *imageData, VimosExtractionTable *extractionTable,
       }
     }
     if (coeffsX != NULL) {
-      pil_free(coeffsX);
+      cpl_free(coeffsX);
       coeffsX = NULL;
     }
     if (coeffsY != NULL) {
-      pil_free(coeffsY);
+      cpl_free(coeffsY);
       coeffsY = NULL;
     }
     
@@ -466,7 +467,7 @@ VmSpStackFF(VimosImage **flatImages, int numFlat,
 
   cpl_msg_info(modName, "Stacking %d flat fields", numFlat);
 
-  outputImages = (VimosImage **) pil_malloc(2*sizeof(VimosImage *));
+  outputImages = (VimosImage **) cpl_malloc(2*sizeof(VimosImage *));
 
   if (readIntDescriptor(extrTable->descs, pilTrnGetKeyword("ZeroOrderFlag"), 
                         &doZero, comment) == VM_FALSE) {
@@ -569,7 +570,7 @@ VmSpStackFF(VimosImage **flatImages, int numFlat,
       cpl_msg_error(modName, "%d unexposed slits found!", j);
       deleteImage(firstOrderIma);
       deleteImage(zeroOrderIma);
-      pil_free(outputImages);
+      cpl_free(outputImages);
       return NULL;
     }
 
