@@ -48,6 +48,7 @@
 #include "vmfit.h"
 #include "vmutils.h"
 #include "vmmosutils.h"
+#include "cpl.h"
 
 
 /*
@@ -86,7 +87,7 @@ modelWavcal(VimosExtractionTable *extractionTable, int lorder)
     row_coefs = newDpoint(nrows);
 
     if (lorder == 0) {
-      buffer = pil_malloc(nrows * sizeof(float));
+      buffer = cpl_malloc(nrows * sizeof(float));
 
       for (i = 0; i <= order; i++) {
 
@@ -121,7 +122,7 @@ modelWavcal(VimosExtractionTable *extractionTable, int lorder)
 
         }
       }
-      pil_free(buffer);
+      cpl_free(buffer);
     }
     else {
 
@@ -169,13 +170,13 @@ modelWavcal(VimosExtractionTable *extractionTable, int lorder)
 
           }
 
-          pil_free(coefs);
+          cpl_free(coefs);
           coefs = NULL;
         }
 
       }
 
-      pil_free(row_coefs);
+      cpl_free(row_coefs);
     }
     slit = slit->next;
   }
@@ -425,7 +426,7 @@ spectralResolution(VimosImage *image, float lambda, double *meanValue,
   xlen = image->xlen;
   ylen = image->ylen;
 
-  buffer = pil_malloc(ylen * sizeof(double));
+  buffer = cpl_malloc(ylen * sizeof(double));
 
   readDoubleDescriptor(image->descs, pilTrnGetKeyword("Crval", 1), &start, 0);
   readDoubleDescriptor(image->descs, pilTrnGetKeyword("Cdelt", 1), &step, 0);
@@ -517,7 +518,7 @@ spectralResolution(VimosImage *image, float lambda, double *meanValue,
   }
 
   if (n == 0) {
-    pil_free(buffer);
+    cpl_free(buffer);
     return EXIT_FAILURE;
   }
 
@@ -531,7 +532,7 @@ spectralResolution(VimosImage *image, float lambda, double *meanValue,
     }
   }
 
-  pil_free(buffer);
+  cpl_free(buffer);
 
   if (m < 3)
     return EXIT_FAILURE;
@@ -589,7 +590,7 @@ int testLineSaturation(VimosImage *image, VimosTable *lineCat)
   readDoubleDescriptor(image->descs, pilTrnGetKeyword("Cdelt", 1), &step, 0);
 
   npix = 2 * radius + 1;
-  profile = pil_calloc(npix, sizeof(float));
+  profile = cpl_calloc(npix, sizeof(float));
 
   for (i = 0; i < numLines; i++) {
 
@@ -683,7 +684,7 @@ distortionsRms(VimosImage *image, VimosTable *lineCat, double fwhm)
 
   radius = ceil(fwhm / step);
   npix = 2 * radius + 1;
-  profile = pil_calloc(npix, sizeof(float));
+  profile = cpl_calloc(npix, sizeof(float));
 
   rms = 0.0;
   allPeaks = 0;
@@ -741,7 +742,7 @@ distortionsRms(VimosImage *image, VimosTable *lineCat, double fwhm)
 */
   }
 
-  pil_free(profile);
+  cpl_free(profile);
 
   if (allPeaks < 10)
     return 0.0;
@@ -801,7 +802,7 @@ distortionsRms_CPL(VimosImage *image, cpl_table *lineCat, double fwhm)
 
   radius = ceil(fwhm / step);
   npix = 2 * radius + 1;
-  profile = pil_calloc(npix, sizeof(float));
+  profile = cpl_calloc(npix, sizeof(float));
 
   rms = 0.0;
   allPeaks = 0;
@@ -859,7 +860,7 @@ distortionsRms_CPL(VimosImage *image, cpl_table *lineCat, double fwhm)
 /+ */
   }
 
-  pil_free(profile);
+  cpl_free(profile);
 
   if (allPeaks < 10)
     return 0.0;
@@ -1102,7 +1103,7 @@ findCentralPosition(VimosImage *image, VimosDescriptor *desc,
     }
   }
 
-  pil_free(region);
+  cpl_free(region);
 
 
   /*
@@ -1161,7 +1162,7 @@ findCentralPosition(VimosImage *image, VimosDescriptor *desc,
 
   peaks = collectPeaks(region, ny, 200, pixWidth, &npeaks);
 
-  pil_free(region);
+  cpl_free(region);
 
   if (npeaks == 0) {
     *deltaX = 0;
@@ -1171,7 +1172,7 @@ findCentralPosition(VimosImage *image, VimosDescriptor *desc,
 
   nlines = lineCat->cols->len;
   wLen = findColInTab(lineCat, "WLEN");
-  lines = pil_malloc(nlines * sizeof(double));
+  lines = cpl_malloc(nlines * sizeof(double));
 
   for (j = 0; j < nlines; j++)
     lines[j] = wLen->colValue->fArray[j];
@@ -1179,8 +1180,8 @@ findCentralPosition(VimosImage *image, VimosDescriptor *desc,
   out = identPeaks(peaks, npeaks, lines, nlines, 
                    min_disp, max_disp, .1, &nident);
 
-  pil_free(peaks);
-  pil_free(lines);
+  cpl_free(peaks);
+  cpl_free(lines);
 
   if (out == NULL) {
     *deltaX = 0;
@@ -1202,9 +1203,9 @@ findCentralPosition(VimosImage *image, VimosDescriptor *desc,
     }
   }
 
-  pil_free(out[0]);
-  pil_free(out[1]);
-  pil_free(out);
+  cpl_free(out[0]);
+  cpl_free(out[1]);
+  cpl_free(out);
 
   if (found)
     return 0;
@@ -1399,7 +1400,7 @@ findSpectrumBorders(VimosFloatArray *profile, double *upper, double *lower,
   */
 
   windowSize = 2*fuzz + 1;
-  window = (float *) pil_malloc(windowSize*sizeof(float));
+  window = (float *) cpl_malloc(windowSize*sizeof(float));
 
  /*  Look for lower edge */
 
@@ -1430,7 +1431,7 @@ findSpectrumBorders(VimosFloatArray *profile, double *upper, double *lower,
     *upper = profile->len - windowSize + fupper;
 /* printf("%f\n",fupper - fuzz); */
   }
-  pil_free(window);
+  cpl_free(window);
 }
 
 
@@ -1484,15 +1485,15 @@ sortByShutterPosition(VimosImage **image, int imageCount,
   * the codes used to identify identical shutter positions.
   */
 
-  shutLow  = (float *)pil_malloc(imageCount * sizeof(float));
-  shutHigh = (float *)pil_malloc(imageCount * sizeof(float));
-  shutPositionGroup = (int *)pil_malloc(imageCount * sizeof(int));
+  shutLow  = (float *)cpl_malloc(imageCount * sizeof(float));
+  shutHigh = (float *)cpl_malloc(imageCount * sizeof(float));
+  shutPositionGroup = (int *)cpl_malloc(imageCount * sizeof(int));
 
   if (!(shutLow && shutHigh && shutPositionGroup)) {
     cpl_msg_debug(modName, "Not enough memory");
-    pil_free(shutLow);
-    pil_free(shutHigh);
-    pil_free(shutPositionGroup);
+    cpl_free(shutLow);
+    cpl_free(shutHigh);
+    cpl_free(shutPositionGroup);
     return NULL;
   }
 
@@ -1504,9 +1505,9 @@ sortByShutterPosition(VimosImage **image, int imageCount,
                         &quadrant, NULL) == VM_FALSE) {
     cpl_msg_debug(modName, "Cannot read descriptor %s", 
                 pilTrnGetKeyword("Quadrant"));
-    pil_free(shutLow);
-    pil_free(shutHigh);
-    pil_free(shutPositionGroup);
+    cpl_free(shutLow);
+    cpl_free(shutHigh);
+    cpl_free(shutPositionGroup);
     return NULL;
   }
 
@@ -1517,9 +1518,9 @@ sortByShutterPosition(VimosImage **image, int imageCount,
                            mshuString, NULL) == VM_FALSE) {
     cpl_msg_debug(modName, "Cannot read descriptor %s", 
                 pilTrnGetKeyword("MshuMode", quadrant));
-    pil_free(shutLow);
-    pil_free(shutHigh);
-    pil_free(shutPositionGroup);
+    cpl_free(shutLow);
+    cpl_free(shutHigh);
+    cpl_free(shutPositionGroup);
     return NULL;
   }
   if (!strncmp(mshuString, "ON", 2))
@@ -1554,9 +1555,9 @@ sortByShutterPosition(VimosImage **image, int imageCount,
                     pilTrnGetKeyword("MshuPosL", quadrant));
 
       if (error) {
-        pil_free(shutLow);
-        pil_free(shutHigh);
-        pil_free(shutPositionGroup);
+        cpl_free(shutLow);
+        cpl_free(shutHigh);
+        cpl_free(shutPositionGroup);
         return NULL;
       }
     }
@@ -1588,13 +1589,13 @@ sortByShutterPosition(VimosImage **image, int imageCount,
   * Count how many images are belonging to each group
   */
 
-    imageCountInGroup = (int *)pil_calloc(nGroups, sizeof(int));
+    imageCountInGroup = (int *)cpl_calloc(nGroups, sizeof(int));
 
     if (!imageCountInGroup) {
       cpl_msg_debug(modName, "Not enough memory");
-      pil_free(shutLow);
-      pil_free(shutHigh);
-      pil_free(shutPositionGroup);
+      cpl_free(shutLow);
+      cpl_free(shutHigh);
+      cpl_free(shutPositionGroup);
       return NULL;
     }
 
@@ -1611,14 +1612,14 @@ sortByShutterPosition(VimosImage **image, int imageCount,
      * Finally, resort array of input images according to group
      */
 
-    holder = (VimosImage **)pil_malloc(imageCount * sizeof(VimosImage *));
+    holder = (VimosImage **)cpl_malloc(imageCount * sizeof(VimosImage *));
 
     if (!holder) {
       cpl_msg_debug(modName, "Not enough memory");
-      pil_free(shutLow);
-      pil_free(shutHigh);
-      pil_free(shutPositionGroup);
-      pil_free(imageCountInGroup);
+      cpl_free(shutLow);
+      cpl_free(shutHigh);
+      cpl_free(shutPositionGroup);
+      cpl_free(imageCountInGroup);
       return NULL;
     }
 
@@ -1634,7 +1635,7 @@ sortByShutterPosition(VimosImage **image, int imageCount,
     for (i = 0; i < imageCount; i++)
       image[i] = holder[i];
     
-    pil_free(holder);
+    cpl_free(holder);
   }
   else {
     /*
@@ -1642,7 +1643,7 @@ sortByShutterPosition(VimosImage **image, int imageCount,
      * no mask shutters
      */
     nGroups = 1;
-    imageCountInGroup = (int *)pil_calloc(nGroups, sizeof(int));
+    imageCountInGroup = (int *)cpl_calloc(nGroups, sizeof(int));
     imageCountInGroup[0] = imageCount;
   }
 
@@ -1650,9 +1651,9 @@ sortByShutterPosition(VimosImage **image, int imageCount,
    * Cleanup
    */
   
-  pil_free(shutLow);
-  pil_free(shutHigh);
-  pil_free(shutPositionGroup);
+  cpl_free(shutLow);
+  cpl_free(shutHigh);
+  cpl_free(shutPositionGroup);
 
  /*
   * Returned:
@@ -1698,14 +1699,14 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
   readIntDescriptor(descs, pilTrnGetKeyword("Quadrant"), &quad, NULL);
 
   len = sizeof(char) * (strlen(namePAF)+7);
-  pafName = (char *)pil_malloc(len);
+  pafName = (char *)cpl_malloc(len);
   if (!pafName)
     return NULL;
 
   sprintf(pafName, "%s-%d.paf", namePAF, quad);
   fp = fopen(pafName, "w");
   if (!fp) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1736,7 +1737,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   desc = findDescriptor(descs, pilTrnGetKeyword("DateObs"));
   if (!desc) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1748,7 +1749,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   desc = findDescriptor(descs, pilTrnGetKeyword("BeamTemperature", quad));
   if (desc == NULL) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1762,7 +1763,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("CurvatureOrd"), 
                         &ord, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1770,7 +1771,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("CurvatureOrdX"), 
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1778,7 +1779,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("CurvatureOrdY"), 
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1792,7 +1793,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
             pilTrnGetKeyword("Curvature", i, j, k))) == NULL) {
           cpl_msg_error(modName, "Cannot read descriptor %s", 
                       pilTrnGetKeyword("Curvature", i, j, k));
-          pil_free(pafName);
+          cpl_free(pafName);
           return NULL;
         }
 
@@ -1810,7 +1811,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("OptDistOrdX"),
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1823,7 +1824,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
           pilTrnGetKeyword("OptDistX", i, j))) == NULL) {
         cpl_msg_error(modName, "Cannot read descriptor %s", 
                     pilTrnGetKeyword("OptDistX", i, j));
-        pil_free(pafName);
+        cpl_free(pafName);
         return NULL;
       }
 
@@ -1835,7 +1836,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("OptDistOrdY"),
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1848,7 +1849,7 @@ createSpectralDistModelsPAF(VimosDescriptor *descs, char *namePAF)
           pilTrnGetKeyword("OptDistY", i, j))) == NULL) {
         cpl_msg_error(modName, "Cannot read descriptor %s", 
                     pilTrnGetKeyword("OptDistY", i, j));
-        pil_free(pafName);
+        cpl_free(pafName);
         return NULL;
       }
 
@@ -1895,14 +1896,14 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
   readIntDescriptor(descs, pilTrnGetKeyword("Quadrant"), &quad, NULL);
 
   len = sizeof(char) * (strlen(namePAF)+7);
-  pafName = (char *)pil_malloc(len);
+  pafName = (char *)cpl_malloc(len);
   if (!pafName)
     return NULL;
 
   sprintf(pafName, "%s-%d.paf", namePAF, quad);
   fp = fopen(pafName, "w");
   if (!fp) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1931,7 +1932,7 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
   */
 
   if ((desc = findDescriptor(descs, "DATE-OBS")) == NULL) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1943,7 +1944,7 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
 
   desc = findDescriptor(descs, pilTrnGetKeyword("BeamTemperature", quad));
   if (desc == NULL) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1957,7 +1958,7 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("DispersionOrd"), 
                         &ord, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1965,7 +1966,7 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("DispersionOrdX"), 
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1973,7 +1974,7 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("DispersionOrdY"), 
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -1987,7 +1988,7 @@ createIdsPAF(VimosDescriptor *descs, char *namePAF)
             pilTrnGetKeyword("Dispersion", i, j, k))) == NULL) {
           cpl_msg_error(modName, "Cannot read descriptor %s", 
                       pilTrnGetKeyword("Dispersion", i, j, k));
-          pil_free(pafName);
+          cpl_free(pafName);
           return NULL;
         }
 
@@ -2044,12 +2045,12 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
   sprintf(filename, "%s_%s_%d.cmf", namePAF, grismName, quad);
 
   len = strlen(filename) + 1;
-  pafName = pil_malloc(len * sizeof(char));
+  pafName = cpl_malloc(len * sizeof(char));
   strcpy(pafName, filename);    /* Need this, because pafName is returned */
 
   fp = fopen(pafName, "w");
   if (!fp) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2061,10 +2062,10 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
   writeStringPAFEntry(fp, (char *)pilTrnGetKeyword("PafHeaderStart"), NULL);
   writeStringPAFEntry(fp, (char *)pilTrnGetKeyword("PafType"),
                       "Configuration");
-  pafName_noext = (char *)pil_malloc(len - 4);
+  pafName_noext = (char *)cpl_malloc(len - 4);
   sprintf(pafName_noext, "%s_%s_%d", namePAF, grismName, quad);
   writeStringPAFEntry(fp, (char *)pilTrnGetKeyword("PafId"), pafName_noext);
-  pil_free(pafName_noext);
+  cpl_free(pafName_noext);
   writeStringPAFEntry(fp, (char *)pilTrnGetKeyword("PafName"), pafName);
   writeStringPAFEntry(fp, (char *)pilTrnGetKeyword("PafDesc"), "");
   writeStringPAFEntry(fp, (char *)pilTrnGetKeyword("PafCrteName"), "");
@@ -2084,7 +2085,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   desc = findDescriptor(descs, pilTrnGetKeyword("DateObs"));
   if (!desc) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2097,7 +2098,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   desc = findDescriptor(descs, pilTrnGetKeyword("BeamTemperature", quad));
   if (!desc) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2111,7 +2112,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("DispersionOrd"), 
                         &ord, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2119,7 +2120,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("DispersionOrdX"), 
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2127,7 +2128,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("DispersionOrdY"), 
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2141,7 +2142,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
             pilTrnGetKeyword("Dispersion", i, j, k))) == NULL) {
           cpl_msg_error(modName, "Cannot read descriptor %s", 
                       pilTrnGetKeyword("Dispersion", i, j, k));
-          pil_free(pafName);
+          cpl_free(pafName);
           return NULL;
         }
 
@@ -2154,7 +2155,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("IdsXrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2162,7 +2163,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("IdsYrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2175,7 +2176,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   desc = findDescriptor(descs, pilTrnGetKeyword("DateObs"));
   if (!desc) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2188,7 +2189,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
   */
 
   if ((desc = findDescriptor(descs, pilTrnGetKeyword("OptTemp", quad))) == 0) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2202,7 +2203,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("OptDistOrdX"),
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2215,7 +2216,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
           pilTrnGetKeyword("OptDistX", i, j))) == NULL) {
         cpl_msg_error(modName, "Cannot read descriptor %s", 
                     pilTrnGetKeyword("OptDistX", i, j));
-        pil_free(pafName);
+        cpl_free(pafName);
         return NULL;
       }
 
@@ -2227,7 +2228,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("OptDistOrdY"),
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2240,7 +2241,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
           pilTrnGetKeyword("OptDistY", i, j))) == NULL) {
         cpl_msg_error(modName, "Cannot read descriptor %s", 
                     pilTrnGetKeyword("OptDistY", i, j));
-        pil_free(pafName);
+        cpl_free(pafName);
         return NULL;
       }
 
@@ -2252,7 +2253,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("OptXrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2260,7 +2261,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("OptYrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2273,7 +2274,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("CurvatureOrd"), 
                         &ord, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2281,7 +2282,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("CurvatureOrdX"), 
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2289,7 +2290,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("CurvatureOrdY"), 
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2303,7 +2304,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
             pilTrnGetKeyword("Curvature", i, j, k))) == NULL) {
           cpl_msg_error(modName, "Cannot read descriptor %s", 
                       pilTrnGetKeyword("Curvature", i, j, k));
-          pil_free(pafName);
+          cpl_free(pafName);
           return NULL;
         }
 
@@ -2316,7 +2317,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("CurvatureXrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2324,7 +2325,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("CurvatureYrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2337,7 +2338,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("ZeroOrdX"),
                         &xOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2350,7 +2351,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
           pilTrnGetKeyword("ZeroX", i, j))) == NULL) {
         cpl_msg_error(modName, "Cannot read descriptor %s", 
                     pilTrnGetKeyword("ZeroX", i, j));
-        pil_free(pafName);
+        cpl_free(pafName);
         return NULL;
       }
 
@@ -2362,7 +2363,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readIntDescriptor(descs, pilTrnGetKeyword("ZeroOrdY"),
                         &yOrd, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2375,7 +2376,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
           pilTrnGetKeyword("ZeroY", i, j))) == NULL) {
         cpl_msg_error(modName, "Cannot read descriptor %s", 
                     pilTrnGetKeyword("ZeroY", i, j));
-        pil_free(pafName);
+        cpl_free(pafName);
         return NULL;
       }
 
@@ -2387,7 +2388,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("ZeroXrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2395,7 +2396,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("ZeroYrms"),
                            &rms, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
@@ -2403,7 +2404,7 @@ createSpectralDistPAF(VimosDescriptor *descs, char *namePAF)
 
   if (readDoubleDescriptor(descs, pilTrnGetKeyword("ZeroOrderWidth"),
                            &pWidth, NULL) == VM_FALSE) {
-    pil_free(pafName);
+    cpl_free(pafName);
     return NULL;
   }
 
