@@ -1145,7 +1145,7 @@ qcWriteValueInt_CPL(char *filnam, int value, const char *name,
     if (descName[i] == '.')
       descName[i] = ' ';
 
-  fits_open_file(&fptr, filnam, READWRITE, &status);
+  ffopen(&fptr, filnam, READWRITE, &status);
   fits_update_key(fptr, TINT, descName, &value, (char *)comment, &status);
   fits_close_file(fptr, &status);
 
@@ -1229,7 +1229,7 @@ qcWriteValueDouble_CPL(char *filnam, double value, const char *name,
     if (descName[i] == '.')
       descName[i] = ' ';
 
-  fits_open_file(&fptr, filnam, READWRITE, &status);
+  ffopen(&fptr, filnam, READWRITE, &status);
   fits_update_key(fptr, TDOUBLE, descName, &value, (char *)comment, &status);
   fits_close_file(fptr, &status);
 
